@@ -12,10 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import TYPE_CHECKING, Any, Tuple, Type, Union
 from uuid import UUID
-from warnings import warn
-
-from bson.py3compat import PY3
 
 """Tools for representing BSON binary data.
 """
@@ -41,7 +39,10 @@ OLD_UUID_SUBTYPE = 3
 """Old BSON binary subtype for a UUID.
 
 :class:`uuid.UUID` instances will automatically be encoded
-by :mod:`bson` using this subtype.
+by :mod:`bson` using this subtype when using
+:data:`UuidRepresentation.PYTHON_LEGACY`,
+:data:`UuidRepresentation.JAVA_LEGACY`, or
+:data:`UuidRepresentation.CSHARP_LEGACY`.
 
 .. versionadded:: 2.1
 """
@@ -49,14 +50,18 @@ by :mod:`bson` using this subtype.
 UUID_SUBTYPE = 4
 """BSON binary subtype for a UUID.
 
-This is the new BSON binary subtype for UUIDs. The
-current default is :data:`OLD_UUID_SUBTYPE`.
-
-.. versionchanged:: 2.1
-   Changed to subtype 4.
+This is the standard BSON binary subtype for UUIDs.
+:class:`uuid.UUID` instances will automatically be encoded
+by :mod:`bson` using this subtype when using
+:data:`UuidRepresentation.STANDARD`.
 """
 
 
+if TYPE_CHECKING:
+    from array import array as _array
+    from mmap import mmap as _mmap
+
+
 class UuidRepresentation:
     UNSPECIFIED = 0
     """An unspecified UUID representation.
@@ -69,7 +74,7 @@ class UuidRepresentation:
     code. When decoding a BSON binary field with a UUID subtype, a
     :class:`~bson.binary.Binary` instance will be returned instead of a
     :class:`uuid.UUID` instance.
-    
+
     See :ref:`unspecified-representation-details` for details.
 
     .. versionadded:: 3.11
@@ -81,7 +86,7 @@ class UuidRepresentation:
     :class:`uuid.UUID` instances will automatically be encoded to
     and decoded from BSON binary, using RFC-4122 byte order with
     binary subtype :data:`UUID_SUBTYPE`.
-    
+
     See :ref:`standard-representation-details` for details.
 
     .. versionadded:: 3.11
@@ -93,7 +98,7 @@ class UuidRepresentation:
     :class:`uuid.UUID` instances will automatically be encoded to
     and decoded from BSON binary, using RFC-4122 byte order with
     binary subtype :data:`OLD_UUID_SUBTYPE`.
-    
+
     See :ref:`python-legacy-representation-details` for details.
 
     .. versionadded:: 3.11
@@ -105,7 +110,7 @@ class UuidRepresentation:
     :class:`uuid.UUID` instances will automatically be encoded to
     and decoded from BSON binary subtype :data:`OLD_UUID_SUBTYPE`,
     using the Java driver's legacy byte order.
-    
+
     See :ref:`java-legacy-representation-details` for details.
 
     .. versionadded:: 3.11
@@ -117,7 +122,7 @@ class UuidRepresentation:
     :class:`uuid.UUID` instances will automatically be encoded to
     and decoded from BSON binary subtype :data:`OLD_UUID_SUBTYPE`,
     using the C# driver's legacy byte order.
-    
+
     See :ref:`csharp-legacy-representation-details` for details.
 
     .. versionadded:: 3.11
@@ -172,6 +177,12 @@ MD5_SUBTYPE = 5
 """BSON binary subtype for an MD5 hash.
 """
 
+COLUMN_SUBTYPE = 7
+"""BSON binary subtype for columns.
+
+.. versionadded:: 4.0
+"""
+
 USER_DEFINED_SUBTYPE = 128
 """BSON binary subtype for any user defined structure.
 """
@@ -205,8 +216,13 @@ class Binary(bytes):
     """
 
     _type_marker = 5
+    __subtype: int
 
-    def __new__(cls, data, subtype=BINARY_SUBTYPE):
+    def __new__(
+        cls: Type["Binary"],
+        data: Union[memoryview, bytes, "_mmap", "_array"],
+        subtype: int = BINARY_SUBTYPE,
+    ) -> "Binary":
         if not isinstance(subtype, int):
             raise TypeError("subtype must be an instance of int")
         if subtype >= 256 or subtype < 0:
@@ -217,7 +233,9 @@ class Binary(bytes):
         return self
 
     @classmethod
-    def from_uuid(cls, uuid, uuid_representation=UuidRepresentation.STANDARD):
+    def from_uuid(
+        cls: Type["Binary"], uuid: UUID, uuid_representation: int = UuidRepresentation.STANDARD
+    ) -> "Binary":
         """Create a BSON Binary object from a Python UUID.
 
         Creates a :class:`~bson.binary.Binary` object from a
@@ -242,7 +260,7 @@ class Binary(bytes):
 
         if uuid_representation not in ALL_UUID_REPRESENTATIONS:
             raise ValueError(
-                "uuid_representation must be a value " "from bson.binary.UuidRepresentation"
+                "uuid_representation must be a value from bson.binary.UuidRepresentation"
             )
 
         if uuid_representation == UuidRepresentation.UNSPECIFIED:
@@ -270,7 +288,7 @@ class Binary(bytes):
 
         return cls(payload, subtype)
 
-    def as_uuid(self, uuid_representation=UuidRepresentation.STANDARD):
+    def as_uuid(self, uuid_representation: int = UuidRepresentation.STANDARD) -> UUID:
         """Create a Python UUID from this BSON Binary object.
 
         Decodes this binary object as a native :class:`uuid.UUID` instance
@@ -288,11 +306,11 @@ class Binary(bytes):
         .. versionadded:: 3.11
         """
         if self.subtype not in ALL_UUID_SUBTYPES:
-            raise ValueError("cannot decode subtype %s as a uuid" % (self.subtype,))
+            raise ValueError(f"cannot decode subtype {self.subtype} as a uuid")
 
         if uuid_representation not in ALL_UUID_REPRESENTATIONS:
             raise ValueError(
-                "uuid_representation must be a value from " "bson.binary.UuidRepresentation"
+                "uuid_representation must be a value from bson.binary.UuidRepresentation"
             )
 
         if uuid_representation == UuidRepresentation.UNSPECIFIED:
@@ -312,23 +330,22 @@ class Binary(bytes):
                 return UUID(bytes=self)
 
         raise ValueError(
-            "cannot decode subtype %s to %s"
-            % (self.subtype, UUID_REPRESENTATION_NAMES[uuid_representation])
+            f"cannot decode subtype {self.subtype} to {UUID_REPRESENTATION_NAMES[uuid_representation]}"
         )
 
     @property
-    def subtype(self):
+    def subtype(self) -> int:
         """Subtype of this binary data."""
         return self.__subtype
 
-    def __getnewargs__(self):
+    def __getnewargs__(self) -> Tuple[bytes, int]:  # type: ignore[override]
         # Work around http://bugs.python.org/issue7382
-        data = super(Binary, self).__getnewargs__()[0]
-        if PY3 and not isinstance(data, bytes):
+        data = super().__getnewargs__()[0]
+        if not isinstance(data, bytes):
             data = data.encode("latin-1")
         return data, self.__subtype
 
-    def __eq__(self, other):
+    def __eq__(self, other: Any) -> bool:
         if isinstance(other, Binary):
             return (self.__subtype, bytes(self)) == (other.subtype, bytes(other))
         # We don't return NotImplemented here because if we did then
@@ -336,101 +353,11 @@ class Binary(bytes):
         # subclass of str...
         return False
 
-    def __hash__(self):
-        return super(Binary, self).__hash__() ^ hash(self.__subtype)
+    def __hash__(self) -> int:
+        return super().__hash__() ^ hash(self.__subtype)
 
-    def __ne__(self, other):
+    def __ne__(self, other: Any) -> bool:
         return not self == other
 
     def __repr__(self):
-        return "Binary(%s, %s)" % (bytes.__repr__(self), self.__subtype)
-
-
-class UUIDLegacy(Binary):
-    """**DEPRECATED** - UUID wrapper to support working with UUIDs stored as
-    PYTHON_LEGACY.
-
-    .. note:: This class has been deprecated and will be removed in
-       PyMongo 4.0. Use :meth:`~bson.binary.Binary.from_uuid` and
-       :meth:`~bson.binary.Binary.as_uuid` with the appropriate
-       :class:`~bson.binary.UuidRepresentation` to handle legacy-formatted
-       UUIDs instead.::
-
-            from bson import Binary, UUIDLegacy, UuidRepresentation
-            import uuid
-
-            my_uuid = uuid.uuid4()
-            legacy_uuid = UUIDLegacy(my_uuid)
-            binary_uuid = Binary.from_uuid(
-                my_uuid, UuidRepresentation.PYTHON_LEGACY)
-
-            assert legacy_uuid == binary_uuid
-            assert legacy_uuid.uuid == binary_uuid.as_uuid(
-                UuidRepresentation.PYTHON_LEGACY)
-
-    .. doctest::
-
-      >>> import uuid
-      >>> from bson.binary import Binary, UUIDLegacy, STANDARD
-      >>> from bson.codec_options import CodecOptions
-      >>> my_uuid = uuid.uuid4()
-      >>> coll = db.get_collection('test',
-      ...                          CodecOptions(uuid_representation=STANDARD))
-      >>> coll.insert_one({'uuid': Binary(my_uuid.bytes, 3)}).inserted_id
-      ObjectId('...')
-      >>> coll.count_documents({'uuid': my_uuid})
-      0
-      >>> coll.count_documents({'uuid': UUIDLegacy(my_uuid)})
-      1
-      >>> coll.find({'uuid': UUIDLegacy(my_uuid)})[0]['uuid']
-      UUID('...')
-      >>>
-      >>> # Convert from subtype 3 to subtype 4
-      >>> doc = coll.find_one({'uuid': UUIDLegacy(my_uuid)})
-      >>> coll.replace_one({"_id": doc["_id"]}, doc).matched_count
-      1
-      >>> coll.count_documents({'uuid': UUIDLegacy(my_uuid)})
-      0
-      >>> coll.count_documents({'uuid': {'$in': [UUIDLegacy(my_uuid), my_uuid]}})
-      1
-      >>> coll.find_one({'uuid': my_uuid})['uuid']
-      UUID('...')
-
-    Raises :exc:`TypeError` if `obj` is not an instance of :class:`~uuid.UUID`.
-
-    :Parameters:
-      - `obj`: An instance of :class:`~uuid.UUID`.
-
-    .. versionchanged:: 3.11
-       Deprecated. The same functionality can be replicated using the
-       :meth:`~Binary.from_uuid` and :meth:`~Binary.to_uuid` methods with
-       :data:`~UuidRepresentation.PYTHON_LEGACY`.
-    .. versionadded:: 2.1
-    """
-
-    def __new__(cls, obj):
-        warn(
-            "The UUIDLegacy class has been deprecated and will be removed "
-            "in PyMongo 4.0. Use the Binary.from_uuid() and Binary.to_uuid() "
-            "with the appropriate UuidRepresentation to handle "
-            "legacy-formatted UUIDs instead.",
-            DeprecationWarning,
-            stacklevel=2,
-        )
-        if not isinstance(obj, UUID):
-            raise TypeError("obj must be an instance of uuid.UUID")
-        self = Binary.__new__(cls, obj.bytes, OLD_UUID_SUBTYPE)
-        self.__uuid = obj
-        return self
-
-    def __getnewargs__(self):
-        # Support copy and deepcopy
-        return (self.__uuid,)
-
-    @property
-    def uuid(self):
-        """UUID instance wrapped by this UUIDLegacy instance."""
-        return self.__uuid
-
-    def __repr__(self):
-        return "UUIDLegacy('%s')" % self.__uuid
+        return f"Binary({bytes.__repr__(self)}, {self.__subtype})"
