@@ -12,9 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""Tools for working with MongoDB `ObjectIds
-<http://dochub.mongodb.org/core/objectids>`_.
-"""
+"""Tools for working with MongoDB ObjectIds."""
 
 import binascii
 import calendar
@@ -44,7 +42,7 @@ def _random_bytes() -> bytes:
     return os.urandom(5)
 
 
-class ObjectId(object):
+class ObjectId:
     """A MongoDB ObjectId."""
 
     _pid = os.getpid()
@@ -88,7 +86,7 @@ class ObjectId(object):
         :Parameters:
           - `oid` (optional): a valid ObjectId.
 
-        .. seealso:: The MongoDB documentation on `ObjectIds`_.
+        .. seealso:: The MongoDB documentation on  `ObjectIds <http://dochub.mongodb.org/core/objectids>`_.
 
         .. versionchanged:: 3.8
            :class:`~bson.objectid.ObjectId` now implements the `ObjectID
@@ -167,7 +165,6 @@ class ObjectId(object):
 
     def __generate(self) -> None:
         """Generate a new value for this ObjectId."""
-
         # 4 bytes current time
         oid = struct.pack(">I", int(time.time()))
 
@@ -203,9 +200,7 @@ class ObjectId(object):
             else:
                 _raise_invalid_id(oid)
         else:
-            raise TypeError(
-                "id must be an instance of (bytes, str, ObjectId), not %s" % (type(oid),)
-            )
+            raise TypeError(f"id must be an instance of (bytes, str, ObjectId), not {type(oid)}")
 
     @property
     def binary(self) -> bytes:
@@ -225,14 +220,14 @@ class ObjectId(object):
         return datetime.datetime.fromtimestamp(timestamp, utc)
 
     def __getstate__(self) -> bytes:
-        """return value of object for pickling.
+        """Return value of object for pickling.
         needed explicitly because __slots__() defined.
         """
         return self.__id
 
     def __setstate__(self, value: Any) -> None:
-        """explicit state set from pickling"""
-        # Provide backwards compatability with OIDs
+        """Explicit state set from pickling"""
+        # Provide backwards compatibility with OIDs
         # pickled with pymongo-1.9 or older.
         if isinstance(value, dict):
             oid = value["_ObjectId__id"]
@@ -250,7 +245,7 @@ class ObjectId(object):
         return binascii.hexlify(self.__id).decode()
 
     def __repr__(self):
-        return "ObjectId('%s')" % (str(self),)
+        return f"ObjectId('{str(self)}')"
 
     def __eq__(self, other: Any) -> bool:
         if isinstance(other, ObjectId):
