@@ -13,6 +13,9 @@
 # limitations under the License.
 
 """Database level operations."""
+from __future__ import annotations
+
+from copy import deepcopy
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -29,7 +32,7 @@ from typing import (
     cast,
 )
 
-from bson.codec_options import DEFAULT_CODEC_OPTIONS, CodecOptions
+from bson.codec_options import DEFAULT_CODEC_OPTIONS
 from bson.dbref import DBRef
 from bson.son import SON
 from bson.timestamp import Timestamp
@@ -38,10 +41,10 @@ from pymongo.aggregation import _DatabaseAggregationCommand
 from pymongo.change_stream import DatabaseChangeStream
 from pymongo.collection import Collection
 from pymongo.command_cursor import CommandCursor
-from pymongo.common import _ecc_coll_name, _ecoc_coll_name, _esc_coll_name
+from pymongo.common import _ecoc_coll_name, _esc_coll_name
 from pymongo.errors import CollectionInvalid, InvalidName
 from pymongo.read_preferences import ReadPreference, _ServerMode
-from pymongo.typings import _CollationIn, _DocumentType, _Pipeline
+from pymongo.typings import _CollationIn, _DocumentType, _DocumentTypeArg, _Pipeline
 
 
 def _check_name(name):
@@ -55,6 +58,7 @@ def _check_name(name):
 
 
 if TYPE_CHECKING:
+    import bson
     import bson.codec_options
     from pymongo.client_session import ClientSession
     from pymongo.mongo_client import MongoClient
@@ -72,10 +76,10 @@ class Database(common.BaseObject, Generic[_DocumentType]):
         self,
         client: "MongoClient[_DocumentType]",
         name: str,
-        codec_options: Optional[CodecOptions] = None,
+        codec_options: Optional[bson.CodecOptions[_DocumentTypeArg]] = None,
         read_preference: Optional[_ServerMode] = None,
-        write_concern: Optional["WriteConcern"] = None,
-        read_concern: Optional["ReadConcern"] = None,
+        write_concern: Optional[WriteConcern] = None,
+        read_concern: Optional[ReadConcern] = None,
     ) -> None:
         """Get a database by client and name.
 
@@ -123,7 +127,7 @@ class Database(common.BaseObject, Generic[_DocumentType]):
 
                db.__my_collection__
         """
-        super(Database, self).__init__(
+        super().__init__(
             codec_options or client.codec_options,
             read_preference or client.read_preference,
             write_concern or client.write_concern,
@@ -152,10 +156,10 @@ class Database(common.BaseObject, Generic[_DocumentType]):
 
     def with_options(
         self,
-        codec_options: Optional[CodecOptions] = None,
+        codec_options: Optional[bson.CodecOptions[_DocumentTypeArg]] = None,
         read_preference: Optional[_ServerMode] = None,
-        write_concern: Optional["WriteConcern"] = None,
-        read_concern: Optional["ReadConcern"] = None,
+        write_concern: Optional[WriteConcern] = None,
+        read_concern: Optional[ReadConcern] = None,
     ) -> "Database[_DocumentType]":
         """Get a clone of this database changing the specified settings.
 
@@ -209,7 +213,7 @@ class Database(common.BaseObject, Generic[_DocumentType]):
         return hash((self.__client, self.__name))
 
     def __repr__(self):
-        return "Database(%r, %r)" % (self.__client, self.__name)
+        return f"Database({self.__client!r}, {self.__name!r})"
 
     def __getattr__(self, name: str) -> Collection[_DocumentType]:
         """Get a collection of this database by name.
@@ -221,8 +225,8 @@ class Database(common.BaseObject, Generic[_DocumentType]):
         """
         if name.startswith("_"):
             raise AttributeError(
-                "Database has no attribute %r. To access the %s"
-                " collection, use database[%r]." % (name, name, name)
+                "Database has no attribute {!r}. To access the {}"
+                " collection, use database[{!r}].".format(name, name, name)
             )
         return self.__getitem__(name)
 
@@ -239,10 +243,10 @@ class Database(common.BaseObject, Generic[_DocumentType]):
     def get_collection(
         self,
         name: str,
-        codec_options: Optional[CodecOptions] = None,
+        codec_options: Optional[bson.CodecOptions[_DocumentTypeArg]] = None,
         read_preference: Optional[_ServerMode] = None,
-        write_concern: Optional["WriteConcern"] = None,
-        read_concern: Optional["ReadConcern"] = None,
+        write_concern: Optional[WriteConcern] = None,
+        read_concern: Optional[ReadConcern] = None,
     ) -> Collection[_DocumentType]:
         """Get a :class:`~pymongo.collection.Collection` with the given name
         and options.
@@ -291,15 +295,37 @@ class Database(common.BaseObject, Generic[_DocumentType]):
             read_concern,
         )
 
+    def _get_encrypted_fields(self, kwargs, coll_name, ask_db):
+        encrypted_fields = kwargs.get("encryptedFields")
+        if encrypted_fields:
+            return deepcopy(encrypted_fields)
+        if (
+            self.client.options.auto_encryption_opts
+            and self.client.options.auto_encryption_opts._encrypted_fields_map
+            and self.client.options.auto_encryption_opts._encrypted_fields_map.get(
+                f"{self.name}.{coll_name}"
+            )
+        ):
+            return deepcopy(
+                self.client.options.auto_encryption_opts._encrypted_fields_map[
+                    f"{self.name}.{coll_name}"
+                ]
+            )
+        if ask_db and self.client.options.auto_encryption_opts:
+            options = self[coll_name].options()
+            if options.get("encryptedFields"):
+                return deepcopy(options["encryptedFields"])
+        return None
+
     @_csot.apply
     def create_collection(
         self,
         name: str,
-        codec_options: Optional[CodecOptions] = None,
+        codec_options: Optional[bson.CodecOptions[_DocumentTypeArg]] = None,
         read_preference: Optional[_ServerMode] = None,
-        write_concern: Optional["WriteConcern"] = None,
-        read_concern: Optional["ReadConcern"] = None,
-        session: Optional["ClientSession"] = None,
+        write_concern: Optional[WriteConcern] = None,
+        read_concern: Optional[ReadConcern] = None,
+        session: Optional[ClientSession] = None,
         check_exists: Optional[bool] = True,
         **kwargs: Any,
     ) -> Collection[_DocumentType]:
@@ -370,7 +396,6 @@ class Database(common.BaseObject, Generic[_DocumentType]):
 
                 {
                   "escCollection": "enxcol_.encryptedCollection.esc",
-                  "eccCollection": "enxcol_.encryptedCollection.ecc",
                   "ecocCollection": "enxcol_.encryptedCollection.ecoc",
                   "fields": [
                       {
@@ -392,9 +417,9 @@ class Database(common.BaseObject, Generic[_DocumentType]):
                 {
                     // key pattern must be {_id: 1}
                     key: <key pattern>, // required
-                    unique: <bool>, // required, must be ‘true’
+                    unique: <bool>, // required, must be `true`
                     name: <string>, // optional, otherwise automatically generated
-                    v: <int>, // optional, must be ‘2’ if provided
+                    v: <int>, // optional, must be `2` if provided
                 }
           - ``changeStreamPreAndPostImages`` (dict): a document with a boolean field ``enabled`` for
             enabling pre- and post-images.
@@ -418,19 +443,10 @@ class Database(common.BaseObject, Generic[_DocumentType]):
         .. _create collection command:
             https://mongodb.com/docs/manual/reference/command/create
         """
-        encrypted_fields = kwargs.get("encryptedFields")
-        if (
-            not encrypted_fields
-            and self.client.options.auto_encryption_opts
-            and self.client.options.auto_encryption_opts._encrypted_fields_map
-        ):
-            encrypted_fields = self.client.options.auto_encryption_opts._encrypted_fields_map.get(
-                "%s.%s" % (self.name, name)
-            )
-            kwargs["encryptedFields"] = encrypted_fields
-
+        encrypted_fields = self._get_encrypted_fields(kwargs, name, False)
         if encrypted_fields:
             common.validate_is_mapping("encryptedFields", encrypted_fields)
+            kwargs["encryptedFields"] = encrypted_fields
 
         clustered_index = kwargs.get("clusteredIndex")
         if clustered_index:
@@ -458,7 +474,7 @@ class Database(common.BaseObject, Generic[_DocumentType]):
             )
 
     def aggregate(
-        self, pipeline: _Pipeline, session: Optional["ClientSession"] = None, **kwargs: Any
+        self, pipeline: _Pipeline, session: Optional[ClientSession] = None, **kwargs: Any
     ) -> CommandCursor[_DocumentType]:
         """Perform a database-level aggregation.
 
@@ -543,7 +559,7 @@ class Database(common.BaseObject, Generic[_DocumentType]):
         batch_size: Optional[int] = None,
         collation: Optional[_CollationIn] = None,
         start_at_operation_time: Optional[Timestamp] = None,
-        session: Optional["ClientSession"] = None,
+        session: Optional[ClientSession] = None,
         start_after: Optional[Mapping[str, Any]] = None,
         comment: Optional[Any] = None,
         full_document_before_change: Optional[str] = None,
@@ -576,14 +592,13 @@ class Database(common.BaseObject, Generic[_DocumentType]):
         .. code-block:: python
 
             try:
-                with db.watch(
-                        [{'$match': {'operationType': 'insert'}}]) as stream:
+                with db.watch([{"$match": {"operationType": "insert"}}]) as stream:
                     for insert_change in stream:
                         print(insert_change)
             except pymongo.errors.PyMongoError:
                 # The ChangeStream encountered an unrecoverable error or the
                 # resume attempt failed to recreate the cursor.
-                logging.error('...')
+                logging.error("...")
 
         For a precise description of the resume process see the
         `change streams specification`_.
@@ -707,7 +722,7 @@ class Database(common.BaseObject, Generic[_DocumentType]):
         allowable_errors: Optional[Sequence[Union[str, int]]] = None,
         read_preference: Optional[_ServerMode] = None,
         codec_options: "Optional[bson.codec_options.CodecOptions[_CodecDocumentType]]" = None,
-        session: Optional["ClientSession"] = None,
+        session: Optional[ClientSession] = None,
         comment: Optional[Any] = None,
         **kwargs: Any,
     ) -> _CodecDocumentType:
@@ -727,10 +742,10 @@ class Database(common.BaseObject, Generic[_DocumentType]):
 
         >>> db.command("buildinfo")
 
-        For a command where the value matters, like ``{collstats:
+        For a command where the value matters, like ``{count:
         collection_name}`` we can do:
 
-        >>> db.command("collstats", collection_name)
+        >>> db.command("count", collection_name)
 
         For commands that take additional arguments we can use
         kwargs. So ``{filemd5: object_id, root: file_root}`` becomes:
@@ -850,7 +865,6 @@ class Database(common.BaseObject, Generic[_DocumentType]):
 
     def _list_collections(self, sock_info, session, read_preference, **kwargs):
         """Internal listCollections helper."""
-
         coll = self.get_collection("$cmd", read_preference=read_preference)
         cmd = SON([("listCollections", 1), ("cursor", {})])
         cmd.update(kwargs)
@@ -871,7 +885,7 @@ class Database(common.BaseObject, Generic[_DocumentType]):
 
     def list_collections(
         self,
-        session: Optional["ClientSession"] = None,
+        session: Optional[ClientSession] = None,
         filter: Optional[Mapping[str, Any]] = None,
         comment: Optional[Any] = None,
         **kwargs: Any,
@@ -912,7 +926,7 @@ class Database(common.BaseObject, Generic[_DocumentType]):
 
     def list_collection_names(
         self,
-        session: Optional["ClientSession"] = None,
+        session: Optional[ClientSession] = None,
         filter: Optional[Mapping[str, Any]] = None,
         comment: Optional[Any] = None,
         **kwargs: Any,
@@ -976,8 +990,8 @@ class Database(common.BaseObject, Generic[_DocumentType]):
     @_csot.apply
     def drop_collection(
         self,
-        name_or_collection: Union[str, Collection],
-        session: Optional["ClientSession"] = None,
+        name_or_collection: Union[str, Collection[_DocumentTypeArg]],
+        session: Optional[ClientSession] = None,
         comment: Optional[Any] = None,
         encrypted_fields: Optional[Mapping[str, Any]] = None,
     ) -> Dict[str, Any]:
@@ -995,7 +1009,6 @@ class Database(common.BaseObject, Generic[_DocumentType]):
 
                 {
                   "escCollection": "enxcol_.encryptedCollection.esc",
-                  "eccCollection": "enxcol_.encryptedCollection.ecc",
                   "ecocCollection": "enxcol_.encryptedCollection.ecoc",
                   "fields": [
                       {
@@ -1037,29 +1050,16 @@ class Database(common.BaseObject, Generic[_DocumentType]):
 
         if not isinstance(name, str):
             raise TypeError("name_or_collection must be an instance of str")
-        full_name = "%s.%s" % (self.name, name)
-        if (
-            not encrypted_fields
-            and self.client.options.auto_encryption_opts
-            and self.client.options.auto_encryption_opts._encrypted_fields_map
-        ):
-            encrypted_fields = self.client.options.auto_encryption_opts._encrypted_fields_map.get(
-                full_name
-            )
-        if not encrypted_fields and self.client.options.auto_encryption_opts:
-            colls = list(
-                self.list_collections(filter={"name": name}, session=session, comment=comment)
-            )
-            if colls and colls[0]["options"].get("encryptedFields"):
-                encrypted_fields = colls[0]["options"]["encryptedFields"]
+        encrypted_fields = self._get_encrypted_fields(
+            {"encryptedFields": encrypted_fields},
+            name,
+            True,
+        )
         if encrypted_fields:
             common.validate_is_mapping("encrypted_fields", encrypted_fields)
             self._drop_helper(
                 _esc_coll_name(encrypted_fields, name), session=session, comment=comment
             )
-            self._drop_helper(
-                _ecc_coll_name(encrypted_fields, name), session=session, comment=comment
-            )
             self._drop_helper(
                 _ecoc_coll_name(encrypted_fields, name), session=session, comment=comment
             )
@@ -1068,10 +1068,10 @@ class Database(common.BaseObject, Generic[_DocumentType]):
 
     def validate_collection(
         self,
-        name_or_collection: Union[str, Collection],
+        name_or_collection: Union[str, Collection[_DocumentTypeArg]],
         scandata: bool = False,
         full: bool = False,
-        session: Optional["ClientSession"] = None,
+        session: Optional[ClientSession] = None,
         background: Optional[bool] = None,
         comment: Optional[Any] = None,
     ) -> Dict[str, Any]:
@@ -1129,14 +1129,14 @@ class Database(common.BaseObject, Generic[_DocumentType]):
         if "result" in result:
             info = result["result"]
             if info.find("exception") != -1 or info.find("corrupt") != -1:
-                raise CollectionInvalid("%s invalid: %s" % (name, info))
+                raise CollectionInvalid(f"{name} invalid: {info}")
         # Sharded results
         elif "raw" in result:
             for _, res in result["raw"].items():
                 if "result" in res:
                     info = res["result"]
                     if info.find("exception") != -1 or info.find("corrupt") != -1:
-                        raise CollectionInvalid("%s invalid: %s" % (name, info))
+                        raise CollectionInvalid(f"{name} invalid: {info}")
                 elif not res.get("valid", False):
                     valid = False
                     break
@@ -1145,7 +1145,7 @@ class Database(common.BaseObject, Generic[_DocumentType]):
             valid = False
 
         if not valid:
-            raise CollectionInvalid("%s invalid: %r" % (name, result))
+            raise CollectionInvalid(f"{name} invalid: {result!r}")
 
         return result
 
@@ -1167,7 +1167,7 @@ class Database(common.BaseObject, Generic[_DocumentType]):
     def dereference(
         self,
         dbref: DBRef,
-        session: Optional["ClientSession"] = None,
+        session: Optional[ClientSession] = None,
         comment: Optional[Any] = None,
         **kwargs: Any,
     ) -> Optional[_DocumentType]:
@@ -1201,7 +1201,7 @@ class Database(common.BaseObject, Generic[_DocumentType]):
         if dbref.database is not None and dbref.database != self.__name:
             raise ValueError(
                 "trying to dereference a DBRef that points to "
-                "another database (%r not %r)" % (dbref.database, self.__name)
+                "another database ({!r} not {!r})".format(dbref.database, self.__name)
             )
         return self[dbref.collection].find_one(
             {"_id": dbref.id}, session=session, comment=comment, **kwargs
