@@ -13,19 +13,26 @@
 # limitations under the License.
 
 """Operation class definitions."""
+from typing import Any, Dict, Generic, List, Mapping, Optional, Sequence, Tuple, Union
 
+from bson.raw_bson import RawBSONDocument
 from pymongo import helpers
 from pymongo.collation import validate_collation_or_none
 from pymongo.common import validate_boolean, validate_is_mapping, validate_list
 from pymongo.helpers import _gen_index_name, _index_document, _index_list
+from pymongo.typings import _CollationIn, _DocumentType, _Pipeline
+
+# Hint supports index name, "myIndex", or list of either strings or index pairs: [('x', 1), ('y', -1), 'z'']
+_IndexList = Sequence[Union[str, Tuple[str, Union[int, str, Mapping[str, Any]]]]]
+_IndexKeyHint = Union[str, _IndexList]
 
 
-class InsertOne(object):
+class InsertOne(Generic[_DocumentType]):
     """Represents an insert_one operation."""
 
     __slots__ = ("_doc",)
 
-    def __init__(self, document):
+    def __init__(self, document: Union[_DocumentType, RawBSONDocument]) -> None:
         """Create an InsertOne instance.
 
         For use with :meth:`~pymongo.collection.Collection.bulk_write`.
@@ -41,23 +48,28 @@ class InsertOne(object):
         bulkobj.add_insert(self._doc)
 
     def __repr__(self):
-        return "InsertOne(%r)" % (self._doc,)
+        return f"InsertOne({self._doc!r})"
 
-    def __eq__(self, other):
+    def __eq__(self, other: Any) -> bool:
         if type(other) == type(self):
             return other._doc == self._doc
         return NotImplemented
 
-    def __ne__(self, other):
+    def __ne__(self, other: Any) -> bool:
         return not self == other
 
 
-class DeleteOne(object):
+class DeleteOne:
     """Represents a delete_one operation."""
 
     __slots__ = ("_filter", "_collation", "_hint")
 
-    def __init__(self, filter, collation=None, hint=None):
+    def __init__(
+        self,
+        filter: Mapping[str, Any],
+        collation: Optional[_CollationIn] = None,
+        hint: Optional[_IndexKeyHint] = None,
+    ) -> None:
         """Create a DeleteOne instance.
 
         For use with :meth:`~pymongo.collection.Collection.bulk_write`.
@@ -65,8 +77,7 @@ class DeleteOne(object):
         :Parameters:
           - `filter`: A query that matches the document to delete.
           - `collation` (optional): An instance of
-            :class:`~pymongo.collation.Collation`. This option is only
-            supported on MongoDB 3.4 and above.
+            :class:`~pymongo.collation.Collation`.
           - `hint` (optional): An index to use to support the query
             predicate specified either by its string name, or in the same
             format as passed to
@@ -93,23 +104,28 @@ class DeleteOne(object):
         bulkobj.add_delete(self._filter, 1, collation=self._collation, hint=self._hint)
 
     def __repr__(self):
-        return "DeleteOne(%r, %r)" % (self._filter, self._collation)
+        return f"DeleteOne({self._filter!r}, {self._collation!r})"
 
-    def __eq__(self, other):
+    def __eq__(self, other: Any) -> bool:
         if type(other) == type(self):
             return (other._filter, other._collation) == (self._filter, self._collation)
         return NotImplemented
 
-    def __ne__(self, other):
+    def __ne__(self, other: Any) -> bool:
         return not self == other
 
 
-class DeleteMany(object):
+class DeleteMany:
     """Represents a delete_many operation."""
 
     __slots__ = ("_filter", "_collation", "_hint")
 
-    def __init__(self, filter, collation=None, hint=None):
+    def __init__(
+        self,
+        filter: Mapping[str, Any],
+        collation: Optional[_CollationIn] = None,
+        hint: Optional[_IndexKeyHint] = None,
+    ) -> None:
         """Create a DeleteMany instance.
 
         For use with :meth:`~pymongo.collection.Collection.bulk_write`.
@@ -117,8 +133,7 @@ class DeleteMany(object):
         :Parameters:
           - `filter`: A query that matches the documents to delete.
           - `collation` (optional): An instance of
-            :class:`~pymongo.collation.Collation`. This option is only
-            supported on MongoDB 3.4 and above.
+            :class:`~pymongo.collation.Collation`.
           - `hint` (optional): An index to use to support the query
             predicate specified either by its string name, or in the same
             format as passed to
@@ -145,23 +160,30 @@ class DeleteMany(object):
         bulkobj.add_delete(self._filter, 0, collation=self._collation, hint=self._hint)
 
     def __repr__(self):
-        return "DeleteMany(%r, %r)" % (self._filter, self._collation)
+        return f"DeleteMany({self._filter!r}, {self._collation!r})"
 
-    def __eq__(self, other):
+    def __eq__(self, other: Any) -> bool:
         if type(other) == type(self):
             return (other._filter, other._collation) == (self._filter, self._collation)
         return NotImplemented
 
-    def __ne__(self, other):
+    def __ne__(self, other: Any) -> bool:
         return not self == other
 
 
-class ReplaceOne(object):
+class ReplaceOne(Generic[_DocumentType]):
     """Represents a replace_one operation."""
 
     __slots__ = ("_filter", "_doc", "_upsert", "_collation", "_hint")
 
-    def __init__(self, filter, replacement, upsert=False, collation=None, hint=None):
+    def __init__(
+        self,
+        filter: Mapping[str, Any],
+        replacement: Union[_DocumentType, RawBSONDocument],
+        upsert: bool = False,
+        collation: Optional[_CollationIn] = None,
+        hint: Optional[_IndexKeyHint] = None,
+    ) -> None:
         """Create a ReplaceOne instance.
 
         For use with :meth:`~pymongo.collection.Collection.bulk_write`.
@@ -172,8 +194,7 @@ class ReplaceOne(object):
           - `upsert` (optional): If ``True``, perform an insert if no documents
             match the filter.
           - `collation` (optional): An instance of
-            :class:`~pymongo.collation.Collation`. This option is only
-            supported on MongoDB 3.4 and above.
+            :class:`~pymongo.collation.Collation`.
           - `hint` (optional): An index to use to support the query
             predicate specified either by its string name, or in the same
             format as passed to
@@ -206,7 +227,7 @@ class ReplaceOne(object):
             self._filter, self._doc, self._upsert, collation=self._collation, hint=self._hint
         )
 
-    def __eq__(self, other):
+    def __eq__(self, other: Any) -> bool:
         if type(other) == type(self):
             return (other._filter, other._doc, other._upsert, other._collation, other._hint) == (
                 self._filter,
@@ -217,11 +238,11 @@ class ReplaceOne(object):
             )
         return NotImplemented
 
-    def __ne__(self, other):
+    def __ne__(self, other: Any) -> bool:
         return not self == other
 
     def __repr__(self):
-        return "%s(%r, %r, %r, %r, %r)" % (
+        return "{}({!r}, {!r}, {!r}, {!r}, {!r})".format(
             self.__class__.__name__,
             self._filter,
             self._doc,
@@ -231,7 +252,7 @@ class ReplaceOne(object):
         )
 
 
-class _UpdateOp(object):
+class _UpdateOp:
     """Private base class for update operations."""
 
     __slots__ = ("_filter", "_doc", "_upsert", "_collation", "_array_filters", "_hint")
@@ -277,7 +298,7 @@ class _UpdateOp(object):
         return not self == other
 
     def __repr__(self):
-        return "%s(%r, %r, %r, %r, %r, %r)" % (
+        return "{}({!r}, {!r}, {!r}, {!r}, {!r}, {!r})".format(
             self.__class__.__name__,
             self._filter,
             self._doc,
@@ -293,7 +314,15 @@ class UpdateOne(_UpdateOp):
 
     __slots__ = ()
 
-    def __init__(self, filter, update, upsert=False, collation=None, array_filters=None, hint=None):
+    def __init__(
+        self,
+        filter: Mapping[str, Any],
+        update: Union[Mapping[str, Any], _Pipeline],
+        upsert: bool = False,
+        collation: Optional[_CollationIn] = None,
+        array_filters: Optional[List[Mapping[str, Any]]] = None,
+        hint: Optional[_IndexKeyHint] = None,
+    ) -> None:
         """Represents an update_one operation.
 
         For use with :meth:`~pymongo.collection.Collection.bulk_write`.
@@ -304,10 +333,9 @@ class UpdateOne(_UpdateOp):
           - `upsert` (optional): If ``True``, perform an insert if no documents
             match the filter.
           - `collation` (optional): An instance of
-            :class:`~pymongo.collation.Collation`. This option is only
-            supported on MongoDB 3.4 and above.
+            :class:`~pymongo.collation.Collation`.
           - `array_filters` (optional): A list of filters specifying which
-            array elements an update should apply. Requires MongoDB 3.6+.
+            array elements an update should apply.
           - `hint` (optional): An index to use to support the query
             predicate specified either by its string name, or in the same
             format as passed to
@@ -324,7 +352,7 @@ class UpdateOne(_UpdateOp):
         .. versionchanged:: 3.5
            Added the `collation` option.
         """
-        super(UpdateOne, self).__init__(filter, update, upsert, collation, array_filters, hint)
+        super().__init__(filter, update, upsert, collation, array_filters, hint)
 
     def _add_to_bulk(self, bulkobj):
         """Add this operation to the _Bulk instance `bulkobj`."""
@@ -344,7 +372,15 @@ class UpdateMany(_UpdateOp):
 
     __slots__ = ()
 
-    def __init__(self, filter, update, upsert=False, collation=None, array_filters=None, hint=None):
+    def __init__(
+        self,
+        filter: Mapping[str, Any],
+        update: Union[Mapping[str, Any], _Pipeline],
+        upsert: bool = False,
+        collation: Optional[_CollationIn] = None,
+        array_filters: Optional[List[Mapping[str, Any]]] = None,
+        hint: Optional[_IndexKeyHint] = None,
+    ) -> None:
         """Create an UpdateMany instance.
 
         For use with :meth:`~pymongo.collection.Collection.bulk_write`.
@@ -355,10 +391,9 @@ class UpdateMany(_UpdateOp):
           - `upsert` (optional): If ``True``, perform an insert if no documents
             match the filter.
           - `collation` (optional): An instance of
-            :class:`~pymongo.collation.Collation`. This option is only
-            supported on MongoDB 3.4 and above.
+            :class:`~pymongo.collation.Collation`.
           - `array_filters` (optional): A list of filters specifying which
-            array elements an update should apply. Requires MongoDB 3.6+.
+            array elements an update should apply.
           - `hint` (optional): An index to use to support the query
             predicate specified either by its string name, or in the same
             format as passed to
@@ -375,7 +410,7 @@ class UpdateMany(_UpdateOp):
         .. versionchanged:: 3.5
            Added the `collation` option.
         """
-        super(UpdateMany, self).__init__(filter, update, upsert, collation, array_filters, hint)
+        super().__init__(filter, update, upsert, collation, array_filters, hint)
 
     def _add_to_bulk(self, bulkobj):
         """Add this operation to the _Bulk instance `bulkobj`."""
@@ -390,17 +425,19 @@ class UpdateMany(_UpdateOp):
         )
 
 
-class IndexModel(object):
+class IndexModel:
     """Represents an index to create."""
 
     __slots__ = ("__document",)
 
-    def __init__(self, keys, **kwargs):
+    def __init__(self, keys: _IndexKeyHint, **kwargs: Any) -> None:
         """Create an Index instance.
 
         For use with :meth:`~pymongo.collection.Collection.create_indexes`.
 
-        Takes either a single key or a list of (key, direction) pairs.
+        Takes either a single key or a list containing (key, direction) pairs
+        or keys.  If no direction is given, :data:`~pymongo.ASCENDING` will
+        be assumed.
         The key(s) must be an instance of :class:`basestring`
         (:class:`str` in python 3), and the direction(s) must be one of
         (:data:`~pymongo.ASCENDING`, :data:`~pymongo.DESCENDING`,
@@ -428,9 +465,9 @@ class IndexModel(object):
             this collection after <int> seconds. The indexed field must
             be a UTC datetime or the data will not expire.
           - `partialFilterExpression`: A document that specifies a filter for
-            a partial index. Requires MongoDB >= 3.2.
+            a partial index.
           - `collation`: An instance of :class:`~pymongo.collation.Collation`
-            that specifies the collation to use in MongoDB >= 3.4.
+            that specifies the collation to use.
           - `wildcardProjection`: Allows users to include or exclude specific
             field paths from a `wildcard index`_ using the { "$**" : 1} key
             pattern. Requires MongoDB >= 4.2.
@@ -442,11 +479,11 @@ class IndexModel(object):
         server version.
 
         :Parameters:
-          - `keys`: a single key or a list of (key, direction)
-            pairs specifying the index to create
+          - `keys`: a single key or a list containing (key, direction) pairs
+             or keys specifying the index to create.
           - `**kwargs` (optional): any additional index creation
             options (see the above list) should be passed as keyword
-            arguments
+            arguments.
 
         .. versionchanged:: 3.11
            Added the ``hidden`` option.
@@ -454,10 +491,10 @@ class IndexModel(object):
            Added the ``partialFilterExpression`` option to support partial
            indexes.
 
-        .. _wildcard index: https://docs.mongodb.com/master/core/index-wildcard/#wildcard-index-core
+        .. _wildcard index: https://mongodb.com/docs/master/core/index-wildcard/
         """
         keys = _index_list(keys)
-        if "name" not in kwargs:
+        if kwargs.get("name") is None:
             kwargs["name"] = _gen_index_name(keys)
         kwargs["key"] = _index_document(keys)
         collation = validate_collation_or_none(kwargs.pop("collation", None))
@@ -466,7 +503,7 @@ class IndexModel(object):
             self.__document["collation"] = collation
 
     @property
-    def document(self):
+    def document(self) -> Dict[str, Any]:
         """An index document suitable for passing to the createIndexes
         command.
         """
