@@ -20,15 +20,10 @@ import traceback
 
 sys.path[0:0] = [""]
 
-from bson.binary import Binary
 from bson.dbref import DBRef
 from bson.objectid import ObjectId
-from bson.py3compat import MAXSIZE, PY3, iteritems
 from bson.son import SON
 
-if PY3:
-    unichr = chr
-
 gen_target = 100
 reduction_attempts = 10
 examples = 5
@@ -60,7 +55,7 @@ def gen_int():
 
 
 def gen_float():
-    return lambda: (random.random() - 0.5) * MAXSIZE
+    return lambda: (random.random() - 0.5) * sys.maxsize
 
 
 def gen_boolean():
@@ -75,16 +70,8 @@ def gen_printable_string(gen_length):
     return lambda: "".join(gen_list(gen_printable_char(), gen_length)())
 
 
-if PY3:
-
-    def gen_char(set=None):
-        return lambda: bytes([random.randint(0, 255)])
-
-
-else:
-
-    def gen_char(set=None):
-        return lambda: chr(random.randint(0, 255))
+def gen_char(set=None):
+    return lambda: bytes([random.randint(0, 255)])
 
 
 def gen_string(gen_length):
@@ -92,11 +79,11 @@ def gen_string(gen_length):
 
 
 def gen_unichar():
-    return lambda: unichr(random.randint(1, 0xFFF))
+    return lambda: chr(random.randint(1, 0xFFF))
 
 
 def gen_unicode(gen_length):
-    return lambda: u"".join([x for x in gen_list(gen_unichar(), gen_length)() if x not in ".$"])
+    return lambda: "".join([x for x in gen_list(gen_unichar(), gen_length)() if x not in ".$"])
 
 
 def gen_list(generator, gen_length):
@@ -129,7 +116,8 @@ def gen_regexp(gen_length):
     # TODO our patterns only consist of one letter.
     # this is because of a bug in CPython's regex equality testing,
     # which I haven't quite tracked down, so I'm just ignoring it...
-    pattern = lambda: u"".join(gen_list(choose_lifted(u"a"), gen_length)())
+    def pattern():
+        return "".join(gen_list(choose_lifted("a"), gen_length)())
 
     def gen_flags():
         flags = 0
@@ -156,16 +144,10 @@ def gen_dbref():
 
 def gen_mongo_value(depth, ref):
 
-    bintype = Binary
-    if PY3:
-        # If we used Binary in python3 tests would fail since we
-        # decode BSON binary subtype 0 to bytes. Testing this with
-        # bytes in python3 makes a lot more sense.
-        bintype = bytes
     choices = [
         gen_unicode(gen_range(0, 50)),
         gen_printable_string(gen_range(0, 50)),
-        my_map(gen_string(gen_range(0, 1000)), bintype),
+        my_map(gen_string(gen_range(0, 1000)), bytes),
         gen_int(),
         gen_float(),
         gen_boolean(),
@@ -204,7 +186,7 @@ def simplify(case):  # TODO this is a hack
             return (True, simplified)
         else:
             # simplify a value
-            simplified_items = list(iteritems(simplified))
+            simplified_items = list(simplified.items())
             if not len(simplified_items):
                 return (False, case)
             (key, value) = random.choice(simplified_items)
@@ -249,9 +231,9 @@ def check(predicate, generator):
         try:
             if not predicate(case):
                 reduction = reduce(case, predicate)
-                counter_examples.append("after %s reductions: %r" % reduction)
+                counter_examples.append("after {} reductions: {!r}".format(*reduction))
         except:
-            counter_examples.append("%r : %s" % (case, traceback.format_exc()))
+            counter_examples.append(f"{case!r} : {traceback.format_exc()}")
     return counter_examples
 
 
