@@ -22,6 +22,7 @@ import sys
 sys.path[0:0] = [""]
 
 from test import unittest
+from test.unified_format import generate_test_classes
 
 from pymongo import MongoClient
 
@@ -41,8 +42,17 @@ def create_test(test_case):
         if not valid:
             self.assertRaises(Exception, MongoClient, uri, connect=False)
         else:
-            client = MongoClient(uri, connect=False)
-            credentials = client._MongoClient__options._credentials
+            props = {}
+            if credential:
+                props = credential["mechanism_properties"] or {}
+                if props.get("REQUEST_TOKEN_CALLBACK"):
+                    props["request_token_callback"] = lambda x, y: 1
+                    del props["REQUEST_TOKEN_CALLBACK"]
+                if props.get("REFRESH_TOKEN_CALLBACK"):
+                    props["refresh_token_callback"] = lambda a, b: 1
+                    del props["REFRESH_TOKEN_CALLBACK"]
+            client = MongoClient(uri, connect=False, authmechanismproperties=props)
+            credentials = client.options.pool_options._credentials
             if credential is None:
                 self.assertIsNone(credentials)
             else:
@@ -57,7 +67,7 @@ def create_test(test_case):
                 expected = credential["mechanism_properties"]
                 if expected is not None:
                     actual = credentials.mechanism_properties
-                    for key, val in expected.items():
+                    for key, _val in expected.items():
                         if "SERVICE_NAME" in expected:
                             self.assertEqual(actual.service_name, expected["SERVICE_NAME"])
                         elif "CANONICALIZE_HOST_NAME" in expected:
@@ -70,8 +80,18 @@ def create_test(test_case):
                             self.assertEqual(
                                 actual.aws_session_token, expected["AWS_SESSION_TOKEN"]
                             )
+                        elif "PROVIDER_NAME" in expected:
+                            self.assertEqual(actual.provider_name, expected["PROVIDER_NAME"])
+                        elif "request_token_callback" in expected:
+                            self.assertEqual(
+                                actual.request_token_callback, expected["request_token_callback"]
+                            )
+                        elif "refresh_token_callback" in expected:
+                            self.assertEqual(
+                                actual.refresh_token_callback, expected["refresh_token_callback"]
+                            )
                         else:
-                            self.fail("Unhandled property: %s" % (key,))
+                            self.fail(f"Unhandled property: {key}")
                 else:
                     if credential["mechanism"] == "MONGODB-AWS":
                         self.assertIsNone(credentials.mechanism_properties.aws_session_token)
@@ -82,7 +102,7 @@ def create_test(test_case):
 
 
 def create_tests():
-    for filename in glob.glob(os.path.join(_TEST_PATH, "*.json")):
+    for filename in glob.glob(os.path.join(_TEST_PATH, "legacy", "*.json")):
         test_suffix, _ = os.path.splitext(os.path.basename(filename))
         with open(filename) as auth_tests:
             test_cases = json.load(auth_tests)["tests"]
@@ -91,11 +111,18 @@ def create_tests():
                     continue
                 test_method = create_test(test_case)
                 name = str(test_case["description"].lower().replace(" ", "_"))
-                setattr(TestAuthSpec, "test_%s_%s" % (test_suffix, name), test_method)
+                setattr(TestAuthSpec, f"test_{test_suffix}_{name}", test_method)
 
 
 create_tests()
 
 
+globals().update(
+    generate_test_classes(
+        os.path.join(_TEST_PATH, "unified"),
+        module=__name__,
+    )
+)
+
 if __name__ == "__main__":
     unittest.main()
