@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright 2009-present MongoDB, Inc.
 #
@@ -38,7 +37,9 @@ import bson
 from bson import (
     BSON,
     EPOCH_AWARE,
+    DatetimeMS,
     Regex,
+    _datetime_to_millis,
     decode,
     decode_all,
     decode_file_iter,
@@ -48,7 +49,7 @@ from bson import (
 )
 from bson.binary import Binary, UuidRepresentation
 from bson.code import Code
-from bson.codec_options import CodecOptions
+from bson.codec_options import CodecOptions, DatetimeConversion
 from bson.dbref import DBRef
 from bson.errors import InvalidBSON, InvalidDocument
 from bson.int64 import Int64
@@ -181,7 +182,7 @@ class TestBSON(unittest.TestCase):
         )
 
     def test_encoding_defaultdict(self):
-        dct = collections.defaultdict(dict, [("foo", "bar")])
+        dct = collections.defaultdict(dict, [("foo", "bar")])  # type: ignore[arg-type]
         encode(dct)
         self.assertEqual(dct, collections.defaultdict(dict, [("foo", "bar")]))
 
@@ -204,35 +205,33 @@ class TestBSON(unittest.TestCase):
         self.assertInvalid(b"\x07\x00\x00\x00\x02a\x00\x78\x56\x34\x12")
         self.assertInvalid(b"\x09\x00\x00\x00\x10a\x00\x05\x00")
         self.assertInvalid(b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00")
-        self.assertInvalid(b"\x13\x00\x00\x00\x02foo\x00" b"\x04\x00\x00\x00bar\x00\x00")
+        self.assertInvalid(b"\x13\x00\x00\x00\x02foo\x00\x04\x00\x00\x00bar\x00\x00")
         self.assertInvalid(
-            b"\x18\x00\x00\x00\x03foo\x00\x0f\x00\x00" b"\x00\x10bar\x00\xff\xff\xff\x7f\x00\x00"
-        )
-        self.assertInvalid(
-            b"\x15\x00\x00\x00\x03foo\x00\x0c" b"\x00\x00\x00\x08bar\x00\x01\x00\x00"
+            b"\x18\x00\x00\x00\x03foo\x00\x0f\x00\x00\x00\x10bar\x00\xff\xff\xff\x7f\x00\x00"
         )
+        self.assertInvalid(b"\x15\x00\x00\x00\x03foo\x00\x0c\x00\x00\x00\x08bar\x00\x01\x00\x00")
         self.assertInvalid(
             b"\x1c\x00\x00\x00\x03foo\x00"
             b"\x12\x00\x00\x00\x02bar\x00"
             b"\x05\x00\x00\x00baz\x00\x00\x00"
         )
-        self.assertInvalid(b"\x10\x00\x00\x00\x02a\x00" b"\x04\x00\x00\x00abc\xff\x00")
+        self.assertInvalid(b"\x10\x00\x00\x00\x02a\x00\x04\x00\x00\x00abc\xff\x00")
 
     def test_bad_string_lengths(self):
-        self.assertInvalid(b"\x0c\x00\x00\x00\x02\x00" b"\x00\x00\x00\x00\x00\x00")
-        self.assertInvalid(b"\x12\x00\x00\x00\x02\x00" b"\xff\xff\xff\xfffoobar\x00\x00")
-        self.assertInvalid(b"\x0c\x00\x00\x00\x0e\x00" b"\x00\x00\x00\x00\x00\x00")
-        self.assertInvalid(b"\x12\x00\x00\x00\x0e\x00" b"\xff\xff\xff\xfffoobar\x00\x00")
+        self.assertInvalid(b"\x0c\x00\x00\x00\x02\x00\x00\x00\x00\x00\x00\x00")
+        self.assertInvalid(b"\x12\x00\x00\x00\x02\x00\xff\xff\xff\xfffoobar\x00\x00")
+        self.assertInvalid(b"\x0c\x00\x00\x00\x0e\x00\x00\x00\x00\x00\x00\x00")
+        self.assertInvalid(b"\x12\x00\x00\x00\x0e\x00\xff\xff\xff\xfffoobar\x00\x00")
         self.assertInvalid(
-            b"\x18\x00\x00\x00\x0c\x00" b"\x00\x00\x00\x00\x00RY\xb5j" b"\xfa[\xd8A\xd6X]\x99\x00"
+            b"\x18\x00\x00\x00\x0c\x00\x00\x00\x00\x00\x00RY\xb5j\xfa[\xd8A\xd6X]\x99\x00"
         )
         self.assertInvalid(
             b"\x1e\x00\x00\x00\x0c\x00"
             b"\xff\xff\xff\xfffoobar\x00"
             b"RY\xb5j\xfa[\xd8A\xd6X]\x99\x00"
         )
-        self.assertInvalid(b"\x0c\x00\x00\x00\r\x00" b"\x00\x00\x00\x00\x00\x00")
-        self.assertInvalid(b"\x0c\x00\x00\x00\r\x00" b"\xff\xff\xff\xff\x00\x00")
+        self.assertInvalid(b"\x0c\x00\x00\x00\r\x00\x00\x00\x00\x00\x00\x00")
+        self.assertInvalid(b"\x0c\x00\x00\x00\r\x00\xff\xff\xff\xff\x00\x00")
         self.assertInvalid(
             b"\x1c\x00\x00\x00\x0f\x00"
             b"\x15\x00\x00\x00\x00\x00"
@@ -312,7 +311,7 @@ class TestBSON(unittest.TestCase):
 
     def test_decode_all_buffer_protocol(self):
         docs = [{"foo": "bar"}, {}]
-        bs = b"".join(map(encode, docs))
+        bs = b"".join(map(encode, docs))  # type: ignore[arg-type]
         self.assertEqual(docs, decode_all(bytearray(bs)))
         self.assertEqual(docs, decode_all(memoryview(bs)))
         self.assertEqual(docs, decode_all(memoryview(b"1" + bs + b"1")[1:-1]))
@@ -370,7 +369,7 @@ class TestBSON(unittest.TestCase):
             ),
         ]
         for i, data in enumerate(bad_bsons):
-            msg = "bad_bson[{}]".format(i)
+            msg = f"bad_bson[{i}]"
             with self.assertRaises(InvalidBSON, msg=msg):
                 decode_all(data)
             with self.assertRaises(InvalidBSON, msg=msg):
@@ -393,9 +392,7 @@ class TestBSON(unittest.TestCase):
     def test_data_timestamp(self):
         self.assertEqual(
             {"test": Timestamp(4, 20)},
-            decode(
-                b"\x13\x00\x00\x00\x11\x74\x65\x73\x74\x00\x14" b"\x00\x00\x00\x04\x00\x00\x00\x00"
-            ),
+            decode(b"\x13\x00\x00\x00\x11\x74\x65\x73\x74\x00\x14\x00\x00\x00\x04\x00\x00\x00\x00"),
         )
 
     def test_basic_encode(self):
@@ -414,29 +411,29 @@ class TestBSON(unittest.TestCase):
         )
         self.assertEqual(
             encode({"mike": 100}),
-            b"\x0F\x00\x00\x00\x10\x6D\x69\x6B\x65\x00\x64\x00" b"\x00\x00\x00",
+            b"\x0F\x00\x00\x00\x10\x6D\x69\x6B\x65\x00\x64\x00\x00\x00\x00",
         )
         self.assertEqual(
             encode({"hello": 1.5}),
-            b"\x14\x00\x00\x00\x01\x68\x65\x6C\x6C\x6F\x00\x00" b"\x00\x00\x00\x00\x00\xF8\x3F\x00",
+            b"\x14\x00\x00\x00\x01\x68\x65\x6C\x6C\x6F\x00\x00\x00\x00\x00\x00\x00\xF8\x3F\x00",
         )
         self.assertEqual(
             encode({"true": True}), b"\x0C\x00\x00\x00\x08\x74\x72\x75\x65\x00\x01\x00"
         )
         self.assertEqual(
-            encode({"false": False}), b"\x0D\x00\x00\x00\x08\x66\x61\x6C\x73\x65\x00\x00" b"\x00"
+            encode({"false": False}), b"\x0D\x00\x00\x00\x08\x66\x61\x6C\x73\x65\x00\x00\x00"
         )
         self.assertEqual(
             encode({"empty": []}),
-            b"\x11\x00\x00\x00\x04\x65\x6D\x70\x74\x79\x00\x05" b"\x00\x00\x00\x00\x00",
+            b"\x11\x00\x00\x00\x04\x65\x6D\x70\x74\x79\x00\x05\x00\x00\x00\x00\x00",
         )
         self.assertEqual(
             encode({"none": {}}),
-            b"\x10\x00\x00\x00\x03\x6E\x6F\x6E\x65\x00\x05\x00" b"\x00\x00\x00\x00",
+            b"\x10\x00\x00\x00\x03\x6E\x6F\x6E\x65\x00\x05\x00\x00\x00\x00\x00",
         )
         self.assertEqual(
             encode({"test": Binary(b"test", 0)}),
-            b"\x14\x00\x00\x00\x05\x74\x65\x73\x74\x00\x04\x00" b"\x00\x00\x00\x74\x65\x73\x74\x00",
+            b"\x14\x00\x00\x00\x05\x74\x65\x73\x74\x00\x04\x00\x00\x00\x00\x74\x65\x73\x74\x00",
         )
         self.assertEqual(
             encode({"test": Binary(b"test", 2)}),
@@ -445,24 +442,24 @@ class TestBSON(unittest.TestCase):
         )
         self.assertEqual(
             encode({"test": Binary(b"test", 128)}),
-            b"\x14\x00\x00\x00\x05\x74\x65\x73\x74\x00\x04\x00" b"\x00\x00\x80\x74\x65\x73\x74\x00",
+            b"\x14\x00\x00\x00\x05\x74\x65\x73\x74\x00\x04\x00\x00\x00\x80\x74\x65\x73\x74\x00",
         )
         self.assertEqual(encode({"test": None}), b"\x0B\x00\x00\x00\x0A\x74\x65\x73\x74\x00\x00")
         self.assertEqual(
             encode({"date": datetime.datetime(2007, 1, 8, 0, 30, 11)}),
-            b"\x13\x00\x00\x00\x09\x64\x61\x74\x65\x00\x38\xBE" b"\x1C\xFF\x0F\x01\x00\x00\x00",
+            b"\x13\x00\x00\x00\x09\x64\x61\x74\x65\x00\x38\xBE\x1C\xFF\x0F\x01\x00\x00\x00",
         )
         self.assertEqual(
             encode({"regex": re.compile(b"a*b", re.IGNORECASE)}),
-            b"\x12\x00\x00\x00\x0B\x72\x65\x67\x65\x78\x00\x61" b"\x2A\x62\x00\x69\x00\x00",
+            b"\x12\x00\x00\x00\x0B\x72\x65\x67\x65\x78\x00\x61\x2A\x62\x00\x69\x00\x00",
         )
         self.assertEqual(
             encode({"$where": Code("test")}),
-            b"\x16\x00\x00\x00\r$where\x00\x05\x00\x00\x00test" b"\x00\x00",
+            b"\x16\x00\x00\x00\r$where\x00\x05\x00\x00\x00test\x00\x00",
         )
         self.assertEqual(
             encode({"$field": Code("function(){ return true;}", scope=None)}),
-            b"+\x00\x00\x00\r$field\x00\x1a\x00\x00\x00" b"function(){ return true;}\x00\x00",
+            b"+\x00\x00\x00\r$field\x00\x1a\x00\x00\x00function(){ return true;}\x00\x00",
         )
         self.assertEqual(
             encode({"$field": Code("return function(){ return x; }", scope={"x": False})}),
@@ -493,10 +490,10 @@ class TestBSON(unittest.TestCase):
 
     def test_unknown_type(self):
         # Repr value differs with major python version
-        part = "type %r for fieldname 'foo'" % (b"\x14",)
+        part = "type {!r} for fieldname 'foo'".format(b"\x14")
         docs = [
             b"\x0e\x00\x00\x00\x14foo\x00\x01\x00\x00\x00\x00",
-            (b"\x16\x00\x00\x00\x04foo\x00\x0c\x00\x00\x00\x140" b"\x00\x01\x00\x00\x00\x00\x00"),
+            (b"\x16\x00\x00\x00\x04foo\x00\x0c\x00\x00\x00\x140\x00\x01\x00\x00\x00\x00\x00"),
             (
                 b" \x00\x00\x00\x04bar\x00\x16\x00\x00\x00\x030\x00\x0e\x00\x00"
                 b"\x00\x14foo\x00\x01\x00\x00\x00\x00\x00\x00"
@@ -518,7 +515,7 @@ class TestBSON(unittest.TestCase):
         # not support creation of the DBPointer type, but will decode
         # DBPointer to DBRef.
 
-        bs = b"\x18\x00\x00\x00\x0c\x00\x01\x00\x00" b"\x00\x00RY\xb5j\xfa[\xd8A\xd6X]\x99\x00"
+        bs = b"\x18\x00\x00\x00\x0c\x00\x01\x00\x00\x00\x00RY\xb5j\xfa[\xd8A\xd6X]\x99\x00"
 
         self.assertEqual({"": DBRef("", ObjectId("5259b56afa5bd841d6585d99"))}, decode(bs))
 
@@ -564,7 +561,9 @@ class TestBSON(unittest.TestCase):
 
     def test_aware_datetime(self):
         aware = datetime.datetime(1993, 4, 4, 2, tzinfo=FixedOffset(555, "SomeZone"))
-        as_utc = (aware - aware.utcoffset()).replace(tzinfo=utc)
+        offset = aware.utcoffset()
+        assert offset is not None
+        as_utc = (aware - offset).replace(tzinfo=utc)
         self.assertEqual(datetime.datetime(1993, 4, 3, 16, 45, tzinfo=utc), as_utc)
         after = decode(encode({"date": aware}), CodecOptions(tz_aware=True))["date"]
         self.assertEqual(utc, after.tzinfo)
@@ -616,7 +615,9 @@ class TestBSON(unittest.TestCase):
 
     def test_naive_decode(self):
         aware = datetime.datetime(1993, 4, 4, 2, tzinfo=FixedOffset(555, "SomeZone"))
-        naive_utc = (aware - aware.utcoffset()).replace(tzinfo=None)
+        offset = aware.utcoffset()
+        assert offset is not None
+        naive_utc = (aware - offset).replace(tzinfo=None)
         self.assertEqual(datetime.datetime(1993, 4, 3, 16, 45), naive_utc)
         after = decode(encode({"date": aware}))["date"]
         self.assertEqual(None, after.tzinfo)
@@ -628,9 +629,9 @@ class TestBSON(unittest.TestCase):
 
     @unittest.skip("Disabled due to http://bugs.python.org/issue25222")
     def test_bad_encode(self):
-        evil_list = {"a": []}
+        evil_list: dict = {"a": []}
         evil_list["a"].append(evil_list)
-        evil_dict = {}
+        evil_dict: dict = {}
         evil_dict["a"] = evil_dict
         for evil_data in [evil_dict, evil_list]:
             self.assertRaises(Exception, encode, evil_data)
@@ -646,7 +647,7 @@ class TestBSON(unittest.TestCase):
         encoded1 = encode({"x": 256})
         decoded1 = decode(encoded1)["x"]
         self.assertEqual(256, decoded1)
-        self.assertEqual(type(256), type(decoded1))
+        self.assertEqual(int, type(decoded1))
 
         encoded2 = encode({"x": Int64(256)})
         decoded2 = decode(encoded2)["x"]
@@ -744,12 +745,12 @@ class TestBSON(unittest.TestCase):
     def test_custom_class(self):
         self.assertIsInstance(decode(encode({})), dict)
         self.assertNotIsInstance(decode(encode({})), SON)
-        self.assertIsInstance(decode(encode({}), CodecOptions(document_class=SON)), SON)
+        self.assertIsInstance(decode(encode({}), CodecOptions(document_class=SON)), SON)  # type: ignore[type-var]
 
-        self.assertEqual(1, decode(encode({"x": 1}), CodecOptions(document_class=SON))["x"])
+        self.assertEqual(1, decode(encode({"x": 1}), CodecOptions(document_class=SON))["x"])  # type: ignore[type-var]
 
         x = encode({"x": [{"y": 1}]})
-        self.assertIsInstance(decode(x, CodecOptions(document_class=SON))["x"][0], SON)
+        self.assertIsInstance(decode(x, CodecOptions(document_class=SON))["x"][0], SON)  # type: ignore[type-var]
 
     def test_subclasses(self):
         # make sure we can serialize subclasses of native Python types.
@@ -772,7 +773,7 @@ class TestBSON(unittest.TestCase):
 
     def test_ordered_dict(self):
         d = OrderedDict([("one", 1), ("two", 2), ("three", 3), ("four", 4)])
-        self.assertEqual(d, decode(encode(d), CodecOptions(document_class=OrderedDict)))
+        self.assertEqual(d, decode(encode(d), CodecOptions(document_class=OrderedDict)))  # type: ignore[type-var]
 
     def test_bson_regex(self):
         # Invalid Python regex, though valid PCRE.
@@ -781,9 +782,7 @@ class TestBSON(unittest.TestCase):
         self.assertEqual(0, bson_re1.flags)
 
         doc1 = {"r": bson_re1}
-        doc1_bson = (
-            b"\x11\x00\x00\x00" b"\x0br\x00[\\w-\\.]\x00\x00" b"\x00"  # document length  # r: regex
-        )  # document terminator
+        doc1_bson = b"\x11\x00\x00\x00\x0br\x00[\\w-\\.]\x00\x00\x00"  # document length  # r: regex  # document terminator
 
         self.assertEqual(doc1_bson, encode(doc1))
         self.assertEqual(doc1, decode(doc1_bson))
@@ -794,9 +793,7 @@ class TestBSON(unittest.TestCase):
 
         doc2_with_re = {"r": re2}
         doc2_with_bson_re = {"r": bson_re2}
-        doc2_bson = (
-            b"\x11\x00\x00\x00" b"\x0br\x00.*\x00imsux\x00" b"\x00"  # document length  # r: regex
-        )  # document terminator
+        doc2_bson = b"\x11\x00\x00\x00\x0br\x00.*\x00imsux\x00\x00"  # document length  # r: regex  # document terminator
 
         self.assertEqual(doc2_bson, encode(doc2_with_re))
         self.assertEqual(doc2_bson, encode(doc2_with_bson_re))
@@ -913,7 +910,7 @@ class TestBSON(unittest.TestCase):
 
     def test_timestamp_highorder_bits(self):
         doc = {"a": Timestamp(0xFFFFFFFF, 0xFFFFFFFF)}
-        doc_bson = b"\x10\x00\x00\x00" b"\x11a\x00\xff\xff\xff\xff\xff\xff\xff\xff" b"\x00"
+        doc_bson = b"\x10\x00\x00\x00\x11a\x00\xff\xff\xff\xff\xff\xff\xff\xff\x00"
         self.assertEqual(doc_bson, encode(doc))
         self.assertEqual(doc, decode(doc_bson))
 
@@ -927,7 +924,7 @@ class TestBSON(unittest.TestCase):
     def test_bson_encode_thread_safe(self):
         def target(i):
             for j in range(1000):
-                my_int = type("MyInt_%s_%s" % (i, j), (int,), {})
+                my_int = type(f"MyInt_{i}_{j}", (int,), {})
                 bson.encode({"my_int": my_int()})
 
         threads = [ExceptionCatchingThread(target=target, args=(i,)) for i in range(3)]
@@ -941,7 +938,7 @@ class TestBSON(unittest.TestCase):
             self.assertIsNone(t.exc)
 
     def test_raise_invalid_document(self):
-        class Wrapper(object):
+        class Wrapper:
             def __init__(self, val):
                 self.val = val
 
@@ -958,7 +955,7 @@ class TestBSON(unittest.TestCase):
 class TestCodecOptions(unittest.TestCase):
     def test_document_class(self):
         self.assertRaises(TypeError, CodecOptions, document_class=object)
-        self.assertIs(SON, CodecOptions(document_class=SON).document_class)
+        self.assertIs(SON, CodecOptions(document_class=SON).document_class)  # type: ignore[type-var]
 
     def test_tz_aware(self):
         self.assertRaises(TypeError, CodecOptions, tz_aware=1)
@@ -981,14 +978,15 @@ class TestCodecOptions(unittest.TestCase):
             "uuid_representation=UuidRepresentation.UNSPECIFIED, "
             "unicode_decode_error_handler='strict', "
             "tzinfo=None, type_registry=TypeRegistry(type_codecs=[], "
-            "fallback_encoder=None))"
+            "fallback_encoder=None), "
+            "datetime_conversion=DatetimeConversion.DATETIME)"
         )
         self.assertEqual(r, repr(CodecOptions()))
 
     def test_decode_all_defaults(self):
         # Test decode_all()'s default document_class is dict and tz_aware is
         # False.
-        doc = {"sub_document": {}, "dt": datetime.datetime.utcnow()}
+        doc = {"sub_document": {}, "dt": datetime.datetime.now(tz=datetime.timezone.utc)}
 
         decoded = bson.decode_all(bson.encode(doc))[0]
         self.assertIsInstance(decoded["sub_document"], dict)
@@ -997,6 +995,28 @@ class TestCodecOptions(unittest.TestCase):
         with self.assertRaisesRegex(ValueError, "cannot encode native uuid"):
             bson.decode_all(bson.encode({"uuid": uuid.uuid4()}))
 
+    def test_decode_all_no_options(self):
+        # Test decode_all()'s default document_class is dict and tz_aware is
+        # False.
+        doc = {"sub_document": {}, "dt": datetime.datetime.now(tz=datetime.timezone.utc)}
+
+        decoded = bson.decode_all(bson.encode(doc), None)[0]
+        self.assertIsInstance(decoded["sub_document"], dict)
+        self.assertIsNone(decoded["dt"].tzinfo)
+
+        doc2 = {"id": Binary.from_uuid(uuid.uuid4())}
+        decoded = bson.decode_all(bson.encode(doc2), None)[0]
+        self.assertIsInstance(decoded["id"], Binary)
+
+    def test_decode_all_kwarg(self):
+        doc = {"a": uuid.uuid4()}
+        opts = CodecOptions(uuid_representation=UuidRepresentation.STANDARD)
+        encoded = encode(doc, codec_options=opts)
+        # Positional codec_options
+        self.assertEqual([doc], decode_all(encoded, opts))
+        # Keyword codec_options
+        self.assertEqual([doc], decode_all(encoded, codec_options=opts))
+
     def test_unicode_decode_error_handler(self):
         enc = encode({"keystr": "foobar"})
 
@@ -1008,7 +1028,10 @@ class TestCodecOptions(unittest.TestCase):
         # Ensure that strict mode raises an error.
         for invalid in [invalid_key, invalid_val, invalid_both]:
             self.assertRaises(
-                InvalidBSON, decode, invalid, CodecOptions(unicode_decode_error_handler="strict")
+                InvalidBSON,
+                decode,
+                invalid,
+                CodecOptions(unicode_decode_error_handler="strict"),
             )
             self.assertRaises(InvalidBSON, decode, invalid, CodecOptions())
             self.assertRaises(InvalidBSON, decode, invalid)
@@ -1029,7 +1052,10 @@ class TestCodecOptions(unittest.TestCase):
         self.assertEqual(dec, {"keystr": "foobar"})
 
         self.assertRaises(
-            InvalidBSON, decode, invalid_both, CodecOptions(unicode_decode_error_handler="junk")
+            InvalidBSON,
+            decode,
+            invalid_both,
+            CodecOptions(unicode_decode_error_handler="junk"),
         )
 
     def round_trip_pickle(self, obj, pickled_with_older):
@@ -1113,6 +1139,186 @@ class TestCodecOptions(unittest.TestCase):
         )
         self.round_trip_pickle(i64, pickled_with_3)
 
+    def test_bson_encode_decode(self) -> None:
+        doc = {"_id": ObjectId()}
+        encoded = bson.encode(doc)
+        decoded = bson.decode(encoded)
+        encoded = bson.encode(decoded)
+        decoded = bson.decode(encoded)
+        # Documents returned from decode are mutable.
+        decoded["new_field"] = 1
+        self.assertTrue(decoded["_id"].generation_time)
+
+
+class TestDatetimeConversion(unittest.TestCase):
+    def test_comps(self):
+        # Tests other timestamp formats.
+        # Test each of the rich comparison methods.
+        pairs = [
+            (DatetimeMS(-1), DatetimeMS(1)),
+            (DatetimeMS(0), DatetimeMS(0)),
+            (DatetimeMS(1), DatetimeMS(-1)),
+        ]
+
+        comp_ops = ["__lt__", "__le__", "__eq__", "__ne__", "__gt__", "__ge__"]
+        for lh, rh in pairs:
+            for op in comp_ops:
+                self.assertEqual(getattr(lh, op)(rh), getattr(lh._value, op)(rh._value))
+
+    def test_class_conversions(self):
+        # Test class conversions.
+        dtr1 = DatetimeMS(1234)
+        dt1 = dtr1.as_datetime()
+        self.assertEqual(dtr1, DatetimeMS(dt1))
+
+        dt2 = datetime.datetime(1969, 1, 1)
+        dtr2 = DatetimeMS(dt2)
+        self.assertEqual(dtr2.as_datetime(), dt2)
+
+        # Test encode and decode without codec options. Expect: DatetimeMS => datetime
+        dtr1 = DatetimeMS(0)
+        enc1 = encode({"x": dtr1})
+        dec1 = decode(enc1)
+        self.assertEqual(dec1["x"], datetime.datetime(1970, 1, 1))
+        self.assertNotEqual(type(dtr1), type(dec1["x"]))
+
+        # Test encode and decode with codec options. Expect: UTCDateimteRaw => DatetimeMS
+        opts1 = CodecOptions(datetime_conversion=DatetimeConversion.DATETIME_MS)
+        enc1 = encode({"x": dtr1})
+        dec1 = decode(enc1, opts1)
+        self.assertEqual(type(dtr1), type(dec1["x"]))
+        self.assertEqual(dtr1, dec1["x"])
+
+        # Expect: datetime => DatetimeMS
+        opts1 = CodecOptions(datetime_conversion=DatetimeConversion.DATETIME_MS)
+        dt1 = datetime.datetime(1970, 1, 1, tzinfo=datetime.timezone.utc)
+        enc1 = encode({"x": dt1})
+        dec1 = decode(enc1, opts1)
+        self.assertEqual(dec1["x"], DatetimeMS(0))
+        self.assertNotEqual(dt1, type(dec1["x"]))
+
+    def test_clamping(self):
+        # Test clamping from below and above.
+        opts1 = CodecOptions(
+            datetime_conversion=DatetimeConversion.DATETIME_CLAMP,
+            tz_aware=True,
+            tzinfo=datetime.timezone.utc,
+        )
+        below = encode({"x": DatetimeMS(_datetime_to_millis(datetime.datetime.min) - 1)})
+        dec_below = decode(below, opts1)
+        self.assertEqual(
+            dec_below["x"], datetime.datetime.min.replace(tzinfo=datetime.timezone.utc)
+        )
+
+        above = encode({"x": DatetimeMS(_datetime_to_millis(datetime.datetime.max) + 1)})
+        dec_above = decode(above, opts1)
+        self.assertEqual(
+            dec_above["x"],
+            datetime.datetime.max.replace(tzinfo=datetime.timezone.utc, microsecond=999000),
+        )
+
+    def test_tz_clamping(self):
+        # Naive clamping to local tz.
+        opts1 = CodecOptions(datetime_conversion=DatetimeConversion.DATETIME_CLAMP, tz_aware=False)
+        below = encode({"x": DatetimeMS(_datetime_to_millis(datetime.datetime.min) - 24 * 60 * 60)})
+
+        dec_below = decode(below, opts1)
+        self.assertEqual(dec_below["x"], datetime.datetime.min)
+
+        above = encode({"x": DatetimeMS(_datetime_to_millis(datetime.datetime.max) + 24 * 60 * 60)})
+        dec_above = decode(above, opts1)
+        self.assertEqual(
+            dec_above["x"],
+            datetime.datetime.max.replace(microsecond=999000),
+        )
+
+        # Aware clamping.
+        opts2 = CodecOptions(datetime_conversion=DatetimeConversion.DATETIME_CLAMP, tz_aware=True)
+        below = encode({"x": DatetimeMS(_datetime_to_millis(datetime.datetime.min) - 24 * 60 * 60)})
+        dec_below = decode(below, opts2)
+        self.assertEqual(
+            dec_below["x"], datetime.datetime.min.replace(tzinfo=datetime.timezone.utc)
+        )
+
+        above = encode({"x": DatetimeMS(_datetime_to_millis(datetime.datetime.max) + 24 * 60 * 60)})
+        dec_above = decode(above, opts2)
+        self.assertEqual(
+            dec_above["x"],
+            datetime.datetime.max.replace(tzinfo=datetime.timezone.utc, microsecond=999000),
+        )
+
+    def test_datetime_auto(self):
+        # Naive auto, in range.
+        opts1 = CodecOptions(datetime_conversion=DatetimeConversion.DATETIME_AUTO)
+        inr = encode({"x": datetime.datetime(1970, 1, 1)}, codec_options=opts1)
+        dec_inr = decode(inr)
+        self.assertEqual(dec_inr["x"], datetime.datetime(1970, 1, 1))
+
+        # Naive auto, below range.
+        below = encode({"x": DatetimeMS(_datetime_to_millis(datetime.datetime.min) - 24 * 60 * 60)})
+        dec_below = decode(below, opts1)
+        self.assertEqual(
+            dec_below["x"], DatetimeMS(_datetime_to_millis(datetime.datetime.min) - 24 * 60 * 60)
+        )
+
+        # Naive auto, above range.
+        above = encode({"x": DatetimeMS(_datetime_to_millis(datetime.datetime.max) + 24 * 60 * 60)})
+        dec_above = decode(above, opts1)
+        self.assertEqual(
+            dec_above["x"],
+            DatetimeMS(_datetime_to_millis(datetime.datetime.max) + 24 * 60 * 60),
+        )
+
+        # Aware auto, in range.
+        opts2 = CodecOptions(
+            datetime_conversion=DatetimeConversion.DATETIME_AUTO,
+            tz_aware=True,
+            tzinfo=datetime.timezone.utc,
+        )
+        inr = encode({"x": datetime.datetime(1970, 1, 1)}, codec_options=opts2)
+        dec_inr = decode(inr)
+        self.assertEqual(dec_inr["x"], datetime.datetime(1970, 1, 1))
+
+        # Aware auto, below range.
+        below = encode({"x": DatetimeMS(_datetime_to_millis(datetime.datetime.min) - 24 * 60 * 60)})
+        dec_below = decode(below, opts2)
+        self.assertEqual(
+            dec_below["x"], DatetimeMS(_datetime_to_millis(datetime.datetime.min) - 24 * 60 * 60)
+        )
+
+        # Aware auto, above range.
+        above = encode({"x": DatetimeMS(_datetime_to_millis(datetime.datetime.max) + 24 * 60 * 60)})
+        dec_above = decode(above, opts2)
+        self.assertEqual(
+            dec_above["x"],
+            DatetimeMS(_datetime_to_millis(datetime.datetime.max) + 24 * 60 * 60),
+        )
+
+    def test_millis_from_datetime_ms(self):
+        # Test 65+ bit integer conversion, expect OverflowError.
+        big_ms = 2**65
+        with self.assertRaises(OverflowError):
+            encode({"x": DatetimeMS(big_ms)})
+
+        # Subclass of DatetimeMS w/ __int__ override, expect an Error.
+        class DatetimeMSOverride(DatetimeMS):
+            def __int__(self):
+                return float(self._value)
+
+        float_ms = DatetimeMSOverride(2)
+        with self.assertRaises(TypeError):
+            encode({"x": float_ms})
+
+
+class TestLongLongToString(unittest.TestCase):
+    def test_long_long_to_string(self):
+        try:
+            from bson import _cbson
+
+            _cbson._test_long_long_to_str()
+        except ImportError:
+            print("_cbson was not imported. Check compilation logs.")
+
 
 if __name__ == "__main__":
     unittest.main()
