@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright 2009-present MongoDB, Inc.
 #
@@ -30,17 +29,14 @@ class TestCode(unittest.TestCase):
         self.assertRaises(TypeError, Code, 5)
         self.assertRaises(TypeError, Code, None)
         self.assertRaises(TypeError, Code, "aoeu", 5)
-        self.assertRaises(TypeError, Code, u"aoeu", 5)
         self.assertTrue(Code("aoeu"))
-        self.assertTrue(Code(u"aoeu"))
         self.assertTrue(Code("aoeu", {}))
-        self.assertTrue(Code(u"aoeu", {}))
 
     def test_read_only(self):
         c = Code("blah")
 
         def set_c():
-            c.scope = 5
+            c.scope = 5  # type: ignore
 
         self.assertRaises(AttributeError, set_c)
 
@@ -60,7 +56,7 @@ class TestCode(unittest.TestCase):
         self.assertEqual(str(with_scope), str(another_scope))
         self.assertEqual({"new_var": 42, "my_var": 5}, another_scope.scope)
         # No error.
-        Code(u"héllø world¡")
+        Code("héllø world¡")
 
     def test_repr(self):
         c = Code("hello world", {})
@@ -70,7 +66,7 @@ class TestCode(unittest.TestCase):
         c = Code("hello world", {"blah": 3})
         self.assertEqual(repr(c), "Code('hello world', {'blah': 3})")
         c = Code("\x08\xFF")
-        self.assertEqual(repr(c), "Code(%s, None)" % (repr("\x08\xFF"),))
+        self.assertEqual(repr(c), "Code({}, None)".format(repr("\x08\xFF")))
 
     def test_equality(self):
         b = Code("hello")
