@@ -148,14 +148,12 @@ class TestCommon(IntegrationTest):
         self.assertTrue(new_coll.insert_one(doc))
         self.assertRaises(OperationFailure, coll.insert_one, doc)
 
-        m = rs_or_single_client(
-            "mongodb://%s/" % (pair,), replicaSet=client_context.replica_set_name
-        )
+        m = rs_or_single_client(f"mongodb://{pair}/", replicaSet=client_context.replica_set_name)
 
         coll = m.pymongo_test.write_concern_test
         self.assertRaises(OperationFailure, coll.insert_one, doc)
         m = rs_or_single_client(
-            "mongodb://%s/?w=0" % (pair,), replicaSet=client_context.replica_set_name
+            f"mongodb://{pair}/?w=0", replicaSet=client_context.replica_set_name
         )
 
         coll = m.pymongo_test.write_concern_test
@@ -163,10 +161,17 @@ class TestCommon(IntegrationTest):
 
         # Equality tests
         direct = connected(single_client(w=0))
-        direct2 = connected(single_client("mongodb://%s/?w=0" % (pair,), **self.credentials))
+        direct2 = connected(single_client(f"mongodb://{pair}/?w=0", **self.credentials))
         self.assertEqual(direct, direct2)
         self.assertFalse(direct != direct2)
 
+    def test_validate_boolean(self):
+        self.db.test.update_one({}, {"$set": {"total": 1}}, upsert=True)
+        with self.assertRaisesRegex(
+            TypeError, "upsert must be True or False, was: upsert={'upsert': True}"
+        ):
+            self.db.test.update_one({}, {"$set": {"total": 1}}, {"upsert": True})  # type: ignore
+
 
 if __name__ == "__main__":
     unittest.main()
