@@ -19,7 +19,7 @@ import sys
 
 sys.path[0:0] = [""]
 
-from test import IntegrationTest, client_context, unittest
+from test import IntegrationTest, unittest
 from test.utils import (
     TestCreator,
     camel_to_snake,
@@ -55,10 +55,10 @@ def check_result(self, expected_result, result):
     if isinstance(result, _WriteResult):
         for res in expected_result:
             prop = camel_to_snake(res)
-            msg = "%s : %r != %r" % (prop, expected_result, result)
+            msg = f"{prop} : {expected_result!r} != {result!r}"
             # SPEC-869: Only BulkWriteResult has upserted_count.
             if prop == "upserted_count" and not isinstance(result, BulkWriteResult):
-                if result.upserted_id is not None:
+                if result.upserted_id is not None:  # type: ignore
                     upserted_count = 1
                 else:
                     upserted_count = 0
@@ -73,14 +73,14 @@ def check_result(self, expected_result, result):
                     ids = expected_result[res]
                     if isinstance(ids, dict):
                         ids = [ids[str(i)] for i in range(len(ids))]
-                    self.assertEqual(ids, result.inserted_ids, msg)
+                    self.assertEqual(ids, result.inserted_ids, msg)  # type: ignore
             elif prop == "upserted_ids":
                 # Convert indexes from strings to integers.
                 ids = expected_result[res]
                 expected_ids = {}
                 for str_index in ids:
                     expected_ids[int(str_index)] = ids[str_index]
-                self.assertEqual(expected_ids, result.upserted_ids, msg)
+                self.assertEqual(expected_ids, result.upserted_ids, msg)  # type: ignore
             else:
                 self.assertEqual(getattr(result, prop), expected_result[res], msg)
 
