@@ -39,13 +39,10 @@ from bson.son import SON
 from pymongo import ASCENDING, DESCENDING
 from pymongo.collation import Collation
 from pymongo.cursor import Cursor, CursorType
-from pymongo.errors import (
-    ConfigurationError,
-    ExecutionTimeout,
-    InvalidOperation,
-    OperationFailure,
-)
+from pymongo.errors import ExecutionTimeout, InvalidOperation, OperationFailure
+from pymongo.operations import _IndexList
 from pymongo.read_concern import ReadConcern
+from pymongo.read_preferences import ReadPreference
 from pymongo.write_concern import WriteConcern
 
 
@@ -61,7 +58,7 @@ class TestCursor(IntegrationTest):
         )
 
         cursor2 = copy.deepcopy(cursor)
-        self.assertEqual(cursor._Cursor__spec, cursor2._Cursor__spec)
+        self.assertEqual(cursor._Cursor__spec, cursor2._Cursor__spec)  # type: ignore
 
     def test_add_remove_option(self):
         cursor = self.db.test.find()
@@ -143,9 +140,9 @@ class TestCursor(IntegrationTest):
         self.assertRaises(TypeError, coll.find().allow_disk_use, "baz")
 
         cursor = coll.find().allow_disk_use(True)
-        self.assertEqual(True, cursor._Cursor__allow_disk_use)
+        self.assertEqual(True, cursor._Cursor__allow_disk_use)  # type: ignore
         cursor = coll.find().allow_disk_use(False)
-        self.assertEqual(False, cursor._Cursor__allow_disk_use)
+        self.assertEqual(False, cursor._Cursor__allow_disk_use)  # type: ignore
 
     def test_max_time_ms(self):
         db = self.db
@@ -159,15 +156,15 @@ class TestCursor(IntegrationTest):
         coll.find().max_time_ms(1)
 
         cursor = coll.find().max_time_ms(999)
-        self.assertEqual(999, cursor._Cursor__max_time_ms)
+        self.assertEqual(999, cursor._Cursor__max_time_ms)  # type: ignore
         cursor = coll.find().max_time_ms(10).max_time_ms(1000)
-        self.assertEqual(1000, cursor._Cursor__max_time_ms)
+        self.assertEqual(1000, cursor._Cursor__max_time_ms)  # type: ignore
 
         cursor = coll.find().max_time_ms(999)
         c2 = cursor.clone()
-        self.assertEqual(999, c2._Cursor__max_time_ms)
-        self.assertTrue("$maxTimeMS" in cursor._Cursor__query_spec())
-        self.assertTrue("$maxTimeMS" in c2._Cursor__query_spec())
+        self.assertEqual(999, c2._Cursor__max_time_ms)  # type: ignore
+        self.assertTrue("$maxTimeMS" in cursor._Cursor__query_spec())  # type: ignore
+        self.assertTrue("$maxTimeMS" in c2._Cursor__query_spec())  # type: ignore
 
         self.assertTrue(coll.find_one(max_time_ms=1000))
 
@@ -222,79 +219,78 @@ class TestCursor(IntegrationTest):
 
         listener = AllowListEventListener("find", "getMore")
         coll = rs_or_single_client(event_listeners=[listener])[self.db.name].pymongo_test
-        results = listener.results
 
         # Tailable_await defaults.
         list(coll.find(cursor_type=CursorType.TAILABLE_AWAIT))
         # find
-        self.assertFalse("maxTimeMS" in results["started"][0].command)
+        self.assertFalse("maxTimeMS" in listener.started_events[0].command)
         # getMore
-        self.assertFalse("maxTimeMS" in results["started"][1].command)
-        results.clear()
+        self.assertFalse("maxTimeMS" in listener.started_events[1].command)
+        listener.reset()
 
         # Tailable_await with max_await_time_ms set.
         list(coll.find(cursor_type=CursorType.TAILABLE_AWAIT).max_await_time_ms(99))
         # find
-        self.assertEqual("find", results["started"][0].command_name)
-        self.assertFalse("maxTimeMS" in results["started"][0].command)
+        self.assertEqual("find", listener.started_events[0].command_name)
+        self.assertFalse("maxTimeMS" in listener.started_events[0].command)
         # getMore
-        self.assertEqual("getMore", results["started"][1].command_name)
-        self.assertTrue("maxTimeMS" in results["started"][1].command)
-        self.assertEqual(99, results["started"][1].command["maxTimeMS"])
-        results.clear()
+        self.assertEqual("getMore", listener.started_events[1].command_name)
+        self.assertTrue("maxTimeMS" in listener.started_events[1].command)
+        self.assertEqual(99, listener.started_events[1].command["maxTimeMS"])
+        listener.reset()
 
         # Tailable_await with max_time_ms
         list(coll.find(cursor_type=CursorType.TAILABLE_AWAIT).max_time_ms(99))
         # find
-        self.assertEqual("find", results["started"][0].command_name)
-        self.assertTrue("maxTimeMS" in results["started"][0].command)
-        self.assertEqual(99, results["started"][0].command["maxTimeMS"])
+        self.assertEqual("find", listener.started_events[0].command_name)
+        self.assertTrue("maxTimeMS" in listener.started_events[0].command)
+        self.assertEqual(99, listener.started_events[0].command["maxTimeMS"])
         # getMore
-        self.assertEqual("getMore", results["started"][1].command_name)
-        self.assertFalse("maxTimeMS" in results["started"][1].command)
-        results.clear()
+        self.assertEqual("getMore", listener.started_events[1].command_name)
+        self.assertFalse("maxTimeMS" in listener.started_events[1].command)
+        listener.reset()
 
         # Tailable_await with both max_time_ms and max_await_time_ms
         list(coll.find(cursor_type=CursorType.TAILABLE_AWAIT).max_time_ms(99).max_await_time_ms(99))
         # find
-        self.assertEqual("find", results["started"][0].command_name)
-        self.assertTrue("maxTimeMS" in results["started"][0].command)
-        self.assertEqual(99, results["started"][0].command["maxTimeMS"])
+        self.assertEqual("find", listener.started_events[0].command_name)
+        self.assertTrue("maxTimeMS" in listener.started_events[0].command)
+        self.assertEqual(99, listener.started_events[0].command["maxTimeMS"])
         # getMore
-        self.assertEqual("getMore", results["started"][1].command_name)
-        self.assertTrue("maxTimeMS" in results["started"][1].command)
-        self.assertEqual(99, results["started"][1].command["maxTimeMS"])
-        results.clear()
+        self.assertEqual("getMore", listener.started_events[1].command_name)
+        self.assertTrue("maxTimeMS" in listener.started_events[1].command)
+        self.assertEqual(99, listener.started_events[1].command["maxTimeMS"])
+        listener.reset()
 
         # Non tailable_await with max_await_time_ms
         list(coll.find(batch_size=1).max_await_time_ms(99))
         # find
-        self.assertEqual("find", results["started"][0].command_name)
-        self.assertFalse("maxTimeMS" in results["started"][0].command)
+        self.assertEqual("find", listener.started_events[0].command_name)
+        self.assertFalse("maxTimeMS" in listener.started_events[0].command)
         # getMore
-        self.assertEqual("getMore", results["started"][1].command_name)
-        self.assertFalse("maxTimeMS" in results["started"][1].command)
-        results.clear()
+        self.assertEqual("getMore", listener.started_events[1].command_name)
+        self.assertFalse("maxTimeMS" in listener.started_events[1].command)
+        listener.reset()
 
         # Non tailable_await with max_time_ms
         list(coll.find(batch_size=1).max_time_ms(99))
         # find
-        self.assertEqual("find", results["started"][0].command_name)
-        self.assertTrue("maxTimeMS" in results["started"][0].command)
-        self.assertEqual(99, results["started"][0].command["maxTimeMS"])
+        self.assertEqual("find", listener.started_events[0].command_name)
+        self.assertTrue("maxTimeMS" in listener.started_events[0].command)
+        self.assertEqual(99, listener.started_events[0].command["maxTimeMS"])
         # getMore
-        self.assertEqual("getMore", results["started"][1].command_name)
-        self.assertFalse("maxTimeMS" in results["started"][1].command)
+        self.assertEqual("getMore", listener.started_events[1].command_name)
+        self.assertFalse("maxTimeMS" in listener.started_events[1].command)
 
         # Non tailable_await with both max_time_ms and max_await_time_ms
         list(coll.find(batch_size=1).max_time_ms(99).max_await_time_ms(88))
         # find
-        self.assertEqual("find", results["started"][0].command_name)
-        self.assertTrue("maxTimeMS" in results["started"][0].command)
-        self.assertEqual(99, results["started"][0].command["maxTimeMS"])
+        self.assertEqual("find", listener.started_events[0].command_name)
+        self.assertTrue("maxTimeMS" in listener.started_events[0].command)
+        self.assertEqual(99, listener.started_events[0].command["maxTimeMS"])
         # getMore
-        self.assertEqual("getMore", results["started"][1].command_name)
-        self.assertFalse("maxTimeMS" in results["started"][1].command)
+        self.assertEqual("getMore", listener.started_events[1].command_name)
+        self.assertFalse("maxTimeMS" in listener.started_events[1].command)
 
     @client_context.require_test_commands
     @client_context.require_no_mongos
@@ -324,8 +320,7 @@ class TestCursor(IntegrationTest):
         for _ in a:
             break
         b = a.explain()
-        # "cursor" pre MongoDB 2.7.6, "executionStats" post
-        self.assertTrue("cursor" in b or "executionStats" in b)
+        self.assertIn("executionStats", b)
 
     def test_explain_with_read_concern(self):
         # Do not add readConcern level to explain.
@@ -334,7 +329,7 @@ class TestCursor(IntegrationTest):
         self.addCleanup(client.close)
         coll = client.pymongo_test.test.with_options(read_concern=ReadConcern(level="local"))
         self.assertTrue(coll.find().explain())
-        started = listener.results["started"]
+        started = listener.started_events
         self.assertEqual(len(started), 1)
         self.assertNotIn("readConcern", started[0].command)
 
@@ -355,7 +350,7 @@ class TestCursor(IntegrationTest):
         )
 
         spec = [("num", DESCENDING)]
-        index = db.test.create_index(spec)
+        _ = db.test.create_index(spec)
 
         first = next(db.test.find())
         self.assertEqual(0, first.get("num"))
@@ -372,6 +367,21 @@ class TestCursor(IntegrationTest):
             break
         self.assertRaises(InvalidOperation, a.hint, spec)
 
+        db.test.drop()
+        db.test.insert_many([{"num": i, "foo": i} for i in range(100)])
+        spec: _IndexList = ["num", ("foo", DESCENDING)]
+        db.test.create_index(spec)
+        first = next(db.test.find().hint(spec))
+        self.assertEqual(0, first.get("num"))
+        self.assertEqual(0, first.get("foo"))
+
+        db.test.drop()
+        db.test.insert_many([{"num": i, "foo": i} for i in range(100)])
+        spec = ["num"]
+        db.test.create_index(spec)
+        first = next(db.test.find().hint(spec))
+        self.assertEqual(0, first.get("num"))
+
     def test_hint_by_name(self):
         db = self.db
         db.test.drop()
@@ -721,6 +731,8 @@ class TestCursor(IntegrationTest):
             (i["a"], i["b"]) for i in db.test.find().sort([("b", DESCENDING), ("a", ASCENDING)])
         ]
         self.assertEqual(result, expected)
+        result = [(i["a"], i["b"]) for i in db.test.find().sort([("b", DESCENDING), "a"])]
+        self.assertEqual(result, expected)
 
         a = db.test.find()
         a.sort("x", ASCENDING)
@@ -763,7 +775,7 @@ class TestCursor(IntegrationTest):
         self.assertEqual([8, 9], [a["x"] for a in cursor])
 
         a = db.test.find()
-        b = a.where("this.x > 3")
+        _ = a.where("this.x > 3")
         for _ in a:
             break
         self.assertRaises(InvalidOperation, a.where, "this.x < 3")
@@ -933,7 +945,7 @@ class TestCursor(IntegrationTest):
         for a, b in zip(count(99), self.db.test.find()[99:]):
             self.assertEqual(a, b["i"])
 
-        for i in self.db.test.find()[1000:]:
+        for _i in self.db.test.find()[1000:]:
             self.fail()
 
         self.assertEqual(5, len(list(self.db.test.find()[20:25])))
@@ -986,7 +998,7 @@ class TestCursor(IntegrationTest):
         self.assertEqual(self.db.test, self.db.test.find().collection)
 
         def set_coll():
-            self.db.test.find().collection = "hello"
+            self.db.test.find().collection = "hello"  # type: ignore
 
         self.assertRaises(AttributeError, set_coll)
 
@@ -1067,7 +1079,7 @@ class TestCursor(IntegrationTest):
 
         def iterate_cursor():
             while cursor.alive:
-                for doc in cursor:
+                for _doc in cursor:
                     pass
 
         t = threading.Thread(target=iterate_cursor)
@@ -1174,7 +1186,6 @@ class TestCursor(IntegrationTest):
         self.client._process_periodic_tasks()
 
         listener = AllowListEventListener("killCursors")
-        results = listener.results
         client = rs_or_single_client(event_listeners=[listener])
         self.addCleanup(client.close)
         coll = client[self.db.name].test_close_kills_cursors
@@ -1183,7 +1194,7 @@ class TestCursor(IntegrationTest):
         docs_inserted = 1000
         coll.insert_many([{"i": i} for i in range(docs_inserted)])
 
-        results.clear()
+        listener.reset()
 
         # Close a cursor while it's still open on the server.
         cursor = coll.find().batch_size(10)
@@ -1192,13 +1203,13 @@ class TestCursor(IntegrationTest):
         cursor.close()
 
         def assertCursorKilled():
-            self.assertEqual(1, len(results["started"]))
-            self.assertEqual("killCursors", results["started"][0].command_name)
-            self.assertEqual(1, len(results["succeeded"]))
-            self.assertEqual("killCursors", results["succeeded"][0].command_name)
+            self.assertEqual(1, len(listener.started_events))
+            self.assertEqual("killCursors", listener.started_events[0].command_name)
+            self.assertEqual(1, len(listener.succeeded_events))
+            self.assertEqual("killCursors", listener.succeeded_events[0].command_name)
 
         assertCursorKilled()
-        results.clear()
+        listener.reset()
 
         # Close a command cursor while it's still open on the server.
         cursor = coll.aggregate([], batchSize=10)
@@ -1209,7 +1220,7 @@ class TestCursor(IntegrationTest):
         if cursor.cursor_id:
             assertCursorKilled()
         else:
-            self.assertEqual(0, len(results["started"]))
+            self.assertEqual(0, len(listener.started_events))
 
     def test_delete_not_initialized(self):
         # Creating a cursor with invalid arguments will not run __init__
@@ -1221,17 +1232,23 @@ class TestCursor(IntegrationTest):
         listener = AllowListEventListener("find", "getMore")
         client = rs_or_single_client(event_listeners=[listener])
         self.addCleanup(client.close)
-        coll = client[self.db.name].test
+        # We never send primary read preference so override the default.
+        coll = client[self.db.name].get_collection(
+            "test", read_preference=ReadPreference.PRIMARY_PREFERRED
+        )
 
         coll.delete_many({})
         coll.insert_many([{} for _ in range(5)])
         self.addCleanup(coll.drop)
 
         list(coll.find(batch_size=3))
-        started = listener.results["started"]
+        started = listener.started_events
         self.assertEqual(2, len(started))
         self.assertEqual("find", started[0].command_name)
-        self.assertIn("$readPreference", started[0].command)
+        if client_context.is_rs or client_context.is_mongos:
+            self.assertIn("$readPreference", started[0].command)
+        else:
+            self.assertNotIn("$readPreference", started[0].command)
         self.assertEqual("getMore", started[1].command_name)
         self.assertNotIn("$readPreference", started[1].command)
 
@@ -1260,13 +1277,13 @@ class TestRawBatchCursor(IntegrationTest):
                 batches = list(
                     client[self.db.name].test.find_raw_batches(session=session).sort("_id")
                 )
-                cmd = listener.results["started"][0]
+                cmd = listener.started_events[0]
                 self.assertEqual(cmd.command_name, "find")
                 self.assertIn("$clusterTime", cmd.command)
                 self.assertEqual(cmd.command["startTransaction"], True)
                 self.assertEqual(cmd.command["txnNumber"], 1)
                 # Ensure we update $clusterTime from the command response.
-                last_cmd = listener.results["succeeded"][-1]
+                last_cmd = listener.succeeded_events[-1]
                 self.assertEqual(
                     last_cmd.reply["$clusterTime"]["clusterTime"],
                     session.cluster_time["clusterTime"],
@@ -1292,8 +1309,8 @@ class TestRawBatchCursor(IntegrationTest):
 
         self.assertEqual(1, len(batches))
         self.assertEqual(docs, decode_all(batches[0]))
-        self.assertEqual(len(listener.results["started"]), 2)
-        for cmd in listener.results["started"]:
+        self.assertEqual(len(listener.started_events), 2)
+        for cmd in listener.started_events:
             self.assertEqual(cmd.command_name, "find")
 
     @client_context.require_version_min(5, 0, 0)
@@ -1313,7 +1330,7 @@ class TestRawBatchCursor(IntegrationTest):
         self.assertEqual(1, len(batches))
         self.assertEqual(docs, decode_all(batches[0]))
 
-        find_cmd = listener.results["started"][1].command
+        find_cmd = listener.started_events[1].command
         self.assertEqual(find_cmd["readConcern"]["level"], "snapshot")
         self.assertIsNotNone(find_cmd["readConcern"]["atClusterTime"])
 
@@ -1371,15 +1388,15 @@ class TestRawBatchCursor(IntegrationTest):
         c.drop()
         c.insert_many([{"_id": i} for i in range(10)])
 
-        listener.results.clear()
+        listener.reset()
         cursor = c.find_raw_batches(batch_size=4)
 
         # First raw batch of 4 documents.
         next(cursor)
 
-        started = listener.results["started"][0]
-        succeeded = listener.results["succeeded"][0]
-        self.assertEqual(0, len(listener.results["failed"]))
+        started = listener.started_events[0]
+        succeeded = listener.succeeded_events[0]
+        self.assertEqual(0, len(listener.failed_events))
         self.assertEqual("find", started.command_name)
         self.assertEqual("pymongo_test", started.database_name)
         self.assertEqual("find", succeeded.command_name)
@@ -1390,15 +1407,14 @@ class TestRawBatchCursor(IntegrationTest):
         self.assertEqual(len(csr["firstBatch"]), 1)
         self.assertEqual(decode_all(csr["firstBatch"][0]), [{"_id": i} for i in range(0, 4)])
 
-        listener.results.clear()
+        listener.reset()
 
         # Next raw batch of 4 documents.
         next(cursor)
         try:
-            results = listener.results
-            started = results["started"][0]
-            succeeded = results["succeeded"][0]
-            self.assertEqual(0, len(results["failed"]))
+            started = listener.started_events[0]
+            succeeded = listener.succeeded_events[0]
+            self.assertEqual(0, len(listener.failed_events))
             self.assertEqual("getMore", started.command_name)
             self.assertEqual("pymongo_test", started.database_name)
             self.assertEqual("getMore", succeeded.command_name)
@@ -1414,7 +1430,7 @@ class TestRawBatchCursor(IntegrationTest):
 class TestRawBatchCommandCursor(IntegrationTest):
     @classmethod
     def setUpClass(cls):
-        super(TestRawBatchCommandCursor, cls).setUpClass()
+        super().setUpClass()
 
     def test_aggregate_raw(self):
         c = self.db.test
@@ -1441,13 +1457,13 @@ class TestRawBatchCommandCursor(IntegrationTest):
                         [{"$sort": {"_id": 1}}], session=session
                     )
                 )
-                cmd = listener.results["started"][0]
+                cmd = listener.started_events[0]
                 self.assertEqual(cmd.command_name, "aggregate")
                 self.assertIn("$clusterTime", cmd.command)
                 self.assertEqual(cmd.command["startTransaction"], True)
                 self.assertEqual(cmd.command["txnNumber"], 1)
                 # Ensure we update $clusterTime from the command response.
-                last_cmd = listener.results["succeeded"][-1]
+                last_cmd = listener.succeeded_events[-1]
                 self.assertEqual(
                     last_cmd.reply["$clusterTime"]["clusterTime"],
                     session.cluster_time["clusterTime"],
@@ -1472,8 +1488,8 @@ class TestRawBatchCommandCursor(IntegrationTest):
 
         self.assertEqual(1, len(batches))
         self.assertEqual(docs, decode_all(batches[0]))
-        self.assertEqual(len(listener.results["started"]), 3)
-        cmds = listener.results["started"]
+        self.assertEqual(len(listener.started_events), 3)
+        cmds = listener.started_events
         self.assertEqual(cmds[0].command_name, "aggregate")
         self.assertEqual(cmds[1].command_name, "aggregate")
 
@@ -1494,7 +1510,7 @@ class TestRawBatchCommandCursor(IntegrationTest):
         self.assertEqual(1, len(batches))
         self.assertEqual(docs, decode_all(batches[0]))
 
-        find_cmd = listener.results["started"][1].command
+        find_cmd = listener.started_events[1].command
         self.assertEqual(find_cmd["readConcern"]["level"], "snapshot")
         self.assertIsNotNone(find_cmd["readConcern"]["atClusterTime"])
 
@@ -1535,13 +1551,13 @@ class TestRawBatchCommandCursor(IntegrationTest):
         c.drop()
         c.insert_many([{"_id": i} for i in range(10)])
 
-        listener.results.clear()
+        listener.reset()
         cursor = c.aggregate_raw_batches([{"$sort": {"_id": 1}}], batchSize=4)
 
         # Start cursor, no initial batch.
-        started = listener.results["started"][0]
-        succeeded = listener.results["succeeded"][0]
-        self.assertEqual(0, len(listener.results["failed"]))
+        started = listener.started_events[0]
+        succeeded = listener.succeeded_events[0]
+        self.assertEqual(0, len(listener.failed_events))
         self.assertEqual("aggregate", started.command_name)
         self.assertEqual("pymongo_test", started.database_name)
         self.assertEqual("aggregate", succeeded.command_name)
@@ -1550,15 +1566,14 @@ class TestRawBatchCommandCursor(IntegrationTest):
 
         # First batch is empty.
         self.assertEqual(len(csr["firstBatch"]), 0)
-        listener.results.clear()
+        listener.reset()
 
         # Batches of 4 documents.
         n = 0
         for batch in cursor:
-            results = listener.results
-            started = results["started"][0]
-            succeeded = results["succeeded"][0]
-            self.assertEqual(0, len(results["failed"]))
+            started = listener.started_events[0]
+            succeeded = listener.succeeded_events[0]
+            self.assertEqual(0, len(listener.failed_events))
             self.assertEqual("getMore", started.command_name)
             self.assertEqual("pymongo_test", started.database_name)
             self.assertEqual("getMore", succeeded.command_name)
@@ -1569,7 +1584,7 @@ class TestRawBatchCommandCursor(IntegrationTest):
             self.assertEqual(decode_all(batch), [{"_id": i} for i in range(n, min(n + 4, 10))])
 
             n += 4
-            listener.results.clear()
+            listener.reset()
 
 
 if __name__ == "__main__":
