@@ -16,7 +16,7 @@
 
 import re
 import sys
-from typing import Any, Iterable, List, Mapping
+from typing import Any, Iterable, List, Mapping, Union
 
 sys.path[0:0] = [""]
 
@@ -95,7 +95,7 @@ class TestDatabaseNoConnect(unittest.TestCase):
 
     def test_iteration(self):
         db = self.client.pymongo_test
-        if "PyPy" in sys.version:
+        if "PyPy" in sys.version and sys.version_info < (3, 8, 15):
             msg = "'NoneType' object is not callable"
         else:
             msg = "'Database' object is not iterable"
@@ -137,7 +137,7 @@ class TestDatabase(IntegrationTest):
     def test_repr(self):
         self.assertEqual(
             repr(Database(self.client, "pymongo_test")),
-            "Database(%r, %s)" % (self.client, repr("pymongo_test")),
+            "Database({!r}, {})".format(self.client, repr("pymongo_test")),
         )
 
     def test_create_collection(self):
@@ -193,7 +193,6 @@ class TestDatabase(IntegrationTest):
 
     def test_list_collection_names_filter(self):
         listener = OvertCommandListener()
-        results = listener.results
         client = rs_or_single_client(event_listeners=[listener])
         db = client[self.db.name]
         db.capped.drop()
@@ -201,25 +200,38 @@ class TestDatabase(IntegrationTest):
         db.capped.insert_one({})
         db.non_capped.insert_one({})
         self.addCleanup(client.drop_database, db.name)
-
+        filter: Union[None, dict]
         # Should not send nameOnly.
         for filter in ({"options.capped": True}, {"options.capped": True, "name": "capped"}):
-            results.clear()
+            listener.reset()
             names = db.list_collection_names(filter=filter)
             self.assertEqual(names, ["capped"])
-            self.assertNotIn("nameOnly", results["started"][0].command)
+            self.assertNotIn("nameOnly", listener.started_events[0].command)
 
         # Should send nameOnly (except on 2.6).
-        filter: Any
         for filter in (None, {}, {"name": {"$in": ["capped", "non_capped"]}}):
-            results.clear()
+            listener.reset()
             names = db.list_collection_names(filter=filter)
             self.assertIn("capped", names)
             self.assertIn("non_capped", names)
-            command = results["started"][0].command
+            command = listener.started_events[0].command
             self.assertIn("nameOnly", command)
             self.assertTrue(command["nameOnly"])
 
+    def test_check_exists(self):
+        listener = OvertCommandListener()
+        client = rs_or_single_client(event_listeners=[listener])
+        self.addCleanup(client.close)
+        db = client[self.db.name]
+        db.drop_collection("unique")
+        db.create_collection("unique", check_exists=True)
+        self.assertIn("listCollections", listener.started_command_names())
+        listener.reset()
+        db.drop_collection("unique")
+        db.create_collection("unique", check_exists=False)
+        self.assertTrue(len(listener.started_events) > 0)
+        self.assertNotIn("listCollections", listener.started_command_names())
+
     def test_list_collections(self):
         self.client.drop_database("pymongo_test")
         db = Database(self.client, "pymongo_test")
@@ -250,8 +262,8 @@ class TestDatabase(IntegrationTest):
 
         # Checking if is there any collection which don't exists.
         if (
-            len(set(colls) - set(["test", "test.mike"])) == 0
-            or len(set(colls) - set(["test", "test.mike", "system.indexes"])) == 0
+            len(set(colls) - {"test", "test.mike"}) == 0
+            or len(set(colls) - {"test", "test.mike", "system.indexes"}) == 0
         ):
             self.assertTrue(True)
         else:
@@ -289,10 +301,7 @@ class TestDatabase(IntegrationTest):
         coll_cnt = {}
 
         # Checking if is there any collection which don't exists.
-        if (
-            len(set(colls) - set(["test"])) == 0
-            or len(set(colls) - set(["test", "system.indexes"])) == 0
-        ):
+        if len(set(colls) - {"test"}) == 0 or len(set(colls) - {"test", "system.indexes"}) == 0:
             self.assertTrue(True)
         else:
             self.assertTrue(False)
@@ -423,11 +432,11 @@ class TestDatabase(IntegrationTest):
         db.test.insert_one(SON([("hello", "world"), ("_id", 5)]))
 
         db = self.client.get_database(
-            "pymongo_test", codec_options=CodecOptions(document_class=SON)
+            "pymongo_test", codec_options=CodecOptions(document_class=SON[str, Any])
         )
         cursor = db.test.find()
         for x in cursor:
-            for (k, v) in x.items():
+            for (k, _v) in x.items():
                 self.assertEqual(k, "_id")
                 break
 
@@ -457,7 +466,7 @@ class TestDatabase(IntegrationTest):
 
         db.test.insert_one({"_id": 4, "foo": "bar"})
         db = self.client.get_database(
-            "pymongo_test", codec_options=CodecOptions(document_class=SON)
+            "pymongo_test", codec_options=CodecOptions(document_class=SON[str, Any])
         )
         self.assertEqual(
             SON([("foo", "bar")]), db.dereference(DBRef("test", 4), projection={"_id": False})
