@@ -21,7 +21,7 @@ import sys
 
 sys.path[0:0] = [""]
 
-from test import client_context, unittest
+from test import IntegrationTest, client_context, unittest
 from test.utils import wait_until
 
 from pymongo.common import validate_read_preference_tags
@@ -77,7 +77,7 @@ def create_test(test_case):
             options["tls"] = options.pop("ssl")
         parsed_options = test_case.get("parsed_options")
         # See DRIVERS-1324, unless tls is explicitly set to False we need TLS.
-        needs_tls = not (options and (options.get("ssl") == False or options.get("tls") == False))
+        needs_tls = not (options and (options.get("ssl") is False or options.get("tls") is False))
         if needs_tls and not client_context.tls:
             self.skipTest("this test requires a TLS cluster")
         if not needs_tls and client_context.tls:
@@ -186,5 +186,13 @@ class TestParsingErrors(unittest.TestCase):
         )
 
 
+class TestCaseInsensitive(IntegrationTest):
+    @unittest.skipUnless(_HAVE_DNSPYTHON, "DNS tests require the dnspython module")
+    def test_connect_case_insensitive(self):
+        client = MongoClient("mongodb+srv://TEST1.TEST.BUILD.10GEN.cc/")
+        self.addCleanup(client.close)
+        self.assertGreater(len(client.topology_description.server_descriptions()), 1)
+
+
 if __name__ == "__main__":
     unittest.main()
