@@ -90,10 +90,10 @@ class TestReadPreferencesBase(IntegrationTest):
     @classmethod
     @client_context.require_secondaries_count(1)
     def setUpClass(cls):
-        super(TestReadPreferencesBase, cls).setUpClass()
+        super().setUpClass()
 
     def setUp(self):
-        super(TestReadPreferencesBase, self).setUp()
+        super().setUp()
         # Insert some data so we can use cursors in read_from_which_host
         self.client.pymongo_test.test.drop()
         self.client.get_database(
@@ -119,16 +119,17 @@ class TestReadPreferencesBase(IntegrationTest):
             return "secondary"
         else:
             self.fail(
-                "Cursor used address %s, expected either primary "
-                "%s or secondaries %s" % (address, client.primary, client.secondaries)
+                "Cursor used address {}, expected either primary "
+                "{} or secondaries {}".format(address, client.primary, client.secondaries)
             )
+            return None
 
     def assertReadsFrom(self, expected, **kwargs):
         c = rs_client(**kwargs)
         wait_until(lambda: len(c.nodes - c.arbiters) == client_context.w, "discovered all nodes")
 
         used = self.read_from_which_kind(c)
-        self.assertEqual(expected, used, "Cursor used %s, expected %s" % (used, expected))
+        self.assertEqual(expected, used, f"Cursor used {used}, expected {expected}")
 
 
 class TestSingleSecondaryOk(TestReadPreferencesBase):
@@ -213,7 +214,7 @@ class TestReadPreferences(TestReadPreferencesBase):
         self.assertRaises(ValueError, rs_client, localthresholdms=-1)
 
     def test_zero_latency(self):
-        ping_times = set()
+        ping_times: set = set()
         # Generate unique ping times.
         while len(ping_times) < len(self.client.nodes):
             ping_times.add(random.random())
@@ -255,7 +256,7 @@ class TestReadPreferences(TestReadPreferencesBase):
         # far, and keep reading until we've used all the members or give up.
         # Chance of using only 2 of 3 members 10k times if there's no bug =
         # 3 * (2/3)**10000, very low.
-        used = set()
+        used: set = set()
         i = 0
         while data_members.difference(used) and i < 10000:
             address = self.read_from_which_host(c)
@@ -271,7 +272,7 @@ class TestReadPreferences(TestReadPreferencesBase):
         self.assertFalse(
             not_used,
             "Expected to use primary and all secondaries for mode NEAREST,"
-            " but didn't use %s\nlatencies: %s" % (not_used, latencies),
+            " but didn't use {}\nlatencies: {}".format(not_used, latencies),
         )
 
 
@@ -280,23 +281,21 @@ class ReadPrefTester(MongoClient):
         self.has_read_from = set()
         client_options = client_context.client_options
         client_options.update(kwargs)
-        super(ReadPrefTester, self).__init__(*args, **client_options)
+        super().__init__(*args, **client_options)
 
     @contextlib.contextmanager
     def _socket_for_reads(self, read_preference, session):
-        context = super(ReadPrefTester, self)._socket_for_reads(read_preference, session)
-        with context as (sock_info, secondary_ok):
+        context = super()._socket_for_reads(read_preference, session)
+        with context as (sock_info, read_preference):
             self.record_a_read(sock_info.address)
-            yield sock_info, secondary_ok
+            yield sock_info, read_preference
 
     @contextlib.contextmanager
-    def _secondaryok_for_server(self, read_preference, server, session):
-        context = super(ReadPrefTester, self)._secondaryok_for_server(
-            read_preference, server, session
-        )
-        with context as (sock_info, secondary_ok):
+    def _socket_from_server(self, read_preference, server, session):
+        context = super()._socket_from_server(read_preference, server, session)
+        with context as (sock_info, read_preference):
             self.record_a_read(sock_info.address)
-            yield sock_info, secondary_ok
+            yield sock_info, read_preference
 
     def record_a_read(self, address):
         server = self._get_topology().select_server_by_address(address, 0)
@@ -313,10 +312,13 @@ _PREF_MAP = [
 
 
 class TestCommandAndReadPreference(IntegrationTest):
+    c: ReadPrefTester
+    client_version: Version
+
     @classmethod
     @client_context.require_secondaries_count(1)
     def setUpClass(cls):
-        super(TestCommandAndReadPreference, cls).setUpClass()
+        super().setUpClass()
         cls.c = ReadPrefTester(
             client_context.pair,
             # Ignore round trip times, to test ReadPreference modes only.
@@ -358,7 +360,8 @@ class TestCommandAndReadPreference(IntegrationTest):
                         # Success
                         break
 
-                unused = self.c.secondaries.union(set([self.c.primary])).difference(used)
+                assert self.c.primary is not None
+                unused = self.c.secondaries.union({self.c.primary}).difference(used)
                 if unused:
                     self.fail("Some members not used for NEAREST: %s" % (unused))
             else:
@@ -371,7 +374,10 @@ class TestCommandAndReadPreference(IntegrationTest):
     def _test_coll_helper(self, secondary_ok, coll, meth, *args, **kwargs):
         for mode, server_type in _PREF_MAP:
             new_coll = coll.with_options(read_preference=mode())
-            func = lambda: getattr(new_coll, meth)(*args, **kwargs)
+
+            def func():
+                return getattr(new_coll, meth)(*args, **kwargs)
+
             if secondary_ok:
                 self._test_fn(server_type, func)
             else:
@@ -381,7 +387,10 @@ class TestCommandAndReadPreference(IntegrationTest):
         # Test that the generic command helper obeys the read preference
         # passed to it.
         for mode, server_type in _PREF_MAP:
-            func = lambda: self.c.pymongo_test.command("dbStats", read_preference=mode())
+
+            def func():
+                return self.c.pymongo_test.command("dbStats", read_preference=mode())
+
             self._test_fn(server_type, func)
 
     def test_create_collection(self):
@@ -408,8 +417,10 @@ class TestCommandAndReadPreference(IntegrationTest):
         )
 
     def test_aggregate_write(self):
+        # 5.0 servers support $out on secondaries.
+        secondary_ok = client_context.version.at_least(5, 0)
         self._test_coll_helper(
-            False,
+            secondary_ok,
             self.c.pymongo_test.test,
             "aggregate",
             [{"$project": {"_id": 1}}, {"$out": "agg_write_test"}],
@@ -421,11 +432,11 @@ class TestMovingAverage(unittest.TestCase):
         avg = MovingAverage()
         self.assertIsNone(avg.get())
         avg.add_sample(10)
-        self.assertAlmostEqual(10, avg.get())
+        self.assertAlmostEqual(10, avg.get())  # type: ignore
         avg.add_sample(20)
-        self.assertAlmostEqual(12, avg.get())
+        self.assertAlmostEqual(12, avg.get())  # type: ignore
         avg.add_sample(30)
-        self.assertAlmostEqual(15.6, avg.get())
+        self.assertAlmostEqual(15.6, avg.get())  # type: ignore
 
 
 class TestMongosAndReadPreference(IntegrationTest):
@@ -473,7 +484,8 @@ class TestMongosAndReadPreference(IntegrationTest):
         )
 
         with self.assertRaises(TypeError):
-            Nearest(max_staleness=1.5)  # Float is prohibited.
+            # Float is prohibited.
+            Nearest(max_staleness=1.5)  # type: ignore
 
         with self.assertRaises(ValueError):
             Nearest(max_staleness=0)
@@ -490,7 +502,7 @@ class TestMongosAndReadPreference(IntegrationTest):
         }
         for mode, cls in cases.items():
             with self.assertRaises(TypeError):
-                cls(hedge=[])
+                cls(hedge=[])  # type: ignore
 
             pref = cls(hedge={})
             self.assertEqual(pref.document, {"mode": mode})
@@ -531,16 +543,19 @@ class TestMongosAndReadPreference(IntegrationTest):
         client = rs_client(event_listeners=[listener])
         self.addCleanup(client.close)
         client.admin.command("ping")
-        for mode, cls in cases.items():
+        for _mode, cls in cases.items():
             pref = cls(hedge={"enabled": True})
             coll = client.test.get_collection("test", read_preference=pref)
             listener.reset()
             coll.find_one()
-            started = listener.results["started"]
+            started = listener.started_events
             self.assertEqual(len(started), 1, started)
             cmd = started[0].command
-            self.assertIn("$readPreference", cmd)
-            self.assertEqual(cmd["$readPreference"], pref.document)
+            if client_context.is_rs or client_context.is_mongos:
+                self.assertIn("$readPreference", cmd)
+                self.assertEqual(cmd["$readPreference"], pref.document)
+            else:
+                self.assertNotIn("$readPreference", cmd)
 
     def test_maybe_add_read_preference(self):
 
@@ -609,7 +624,9 @@ class TestMongosAndReadPreference(IntegrationTest):
 
     @client_context.require_mongos
     def test_mongos(self):
-        shard = client_context.client.config.shards.find_one()["host"]
+        res = client_context.client.config.shards.find_one()
+        assert res is not None
+        shard = res["host"]
         num_members = shard.count(",") + 1
         if num_members == 1:
             raise SkipTest("Need a replica set shard to test.")
