@@ -76,14 +76,14 @@ class TestSpec(SpecRunner):
     # TODO: remove this once PYTHON-1948 is done.
     @client_context.require_no_mmap
     def setUpClass(cls):
-        super(TestSpec, cls).setUpClass()
+        super().setUpClass()
 
     def maybe_skip_scenario(self, test):
-        super(TestSpec, self).maybe_skip_scenario(test)
+        super().maybe_skip_scenario(test)
         skip_names = ["listCollectionObjects", "listIndexNames", "listDatabaseObjects"]
         for name in skip_names:
             if name.lower() in test["description"].lower():
-                self.skipTest("PyMongo does not support %s" % (name,))
+                self.skipTest(f"PyMongo does not support {name}")
 
         # Serverless does not support $out and collation.
         if client_context.serverless:
@@ -107,24 +107,27 @@ class TestSpec(SpecRunner):
         """Override a test's collection name to support GridFS tests."""
         if "bucket_name" in scenario_def:
             return scenario_def["bucket_name"]
-        return super(TestSpec, self).get_scenario_coll_name(scenario_def)
+        return super().get_scenario_coll_name(scenario_def)
 
     def setup_scenario(self, scenario_def):
         """Override a test's setup to support GridFS tests."""
         if "bucket_name" in scenario_def:
+            data = scenario_def["data"]
             db_name = self.get_scenario_db_name(scenario_def)
-            db = client_context.client.get_database(
-                db_name, write_concern=WriteConcern(w="majority")
-            )
-            # Create a bucket for the retryable reads GridFS tests.
-            client_context.client.drop_database(db_name)
-            if scenario_def["data"]:
-                data = scenario_def["data"]
-                # Load data.
+            db = client_context.client[db_name]
+            # Create a bucket for the retryable reads GridFS tests with as few
+            # majority writes as possible.
+            wc = WriteConcern(w="majority")
+            if data:
+                db["fs.chunks"].drop()
+                db["fs.files"].drop()
                 db["fs.chunks"].insert_many(data["fs.chunks"])
-                db["fs.files"].insert_many(data["fs.files"])
+                db.get_collection("fs.files", write_concern=wc).insert_many(data["fs.files"])
+            else:
+                db.get_collection("fs.chunks").drop()
+                db.get_collection("fs.files", write_concern=wc).drop()
         else:
-            super(TestSpec, self).setup_scenario(scenario_def)
+            super().setup_scenario(scenario_def)
 
 
 def create_test(scenario_def, test, name):
@@ -159,6 +162,9 @@ class TestPoolPausedError(IntegrationTest):
     @client_context.require_failCommand_blockConnection
     @client_knobs(heartbeat_frequency=0.05, min_heartbeat_interval=0.05)
     def test_pool_paused_error_is_retryable(self):
+        if "PyPy" in sys.version:
+            # Tracked in PYTHON-3519
+            self.skipTest("Test is flakey on PyPy")
         cmap_listener = CMAPListener()
         cmd_listener = OvertCommandListener()
         client = rs_or_single_client(maxPoolSize=1, event_listeners=[cmap_listener, cmd_listener])
@@ -205,12 +211,12 @@ class TestPoolPausedError(IntegrationTest):
         # Connection check out failures are not reflected in command
         # monitoring because we only publish command events _after_ checking
         # out a connection.
-        started = cmd_listener.results["started"]
+        started = cmd_listener.started_events
         msg = pprint.pformat(cmd_listener.results)
         self.assertEqual(3, len(started), msg)
-        succeeded = cmd_listener.results["succeeded"]
+        succeeded = cmd_listener.succeeded_events
         self.assertEqual(2, len(succeeded), msg)
-        failed = cmd_listener.results["failed"]
+        failed = cmd_listener.failed_events
         self.assertEqual(1, len(failed), msg)
 
 
