@@ -26,12 +26,12 @@ from test.utils import (
     ServerAndTopologyEventListener,
     rs_or_single_client,
     server_name_to_type,
-    single_client,
     wait_until,
 )
 
 from bson.json_util import object_hook
-from pymongo import monitoring
+from pymongo import MongoClient, monitoring
+from pymongo.collection import Collection
 from pymongo.common import clean_node
 from pymongo.errors import ConnectionFailure, NotPrimaryError
 from pymongo.hello import Hello
@@ -44,12 +44,12 @@ _TEST_PATH = os.path.join(os.path.dirname(os.path.realpath(__file__)), "sdam_mon
 
 
 def compare_server_descriptions(expected, actual):
-    if (not expected["address"] == "%s:%s" % actual.address) or (
+    if (not expected["address"] == "{}:{}".format(*actual.address)) or (
         not server_name_to_type(expected["type"]) == actual.server_type
     ):
         return False
     expected_hosts = set(expected["arbiters"] + expected["passives"] + expected["hosts"])
-    return expected_hosts == set("%s:%s" % s for s in actual.all_hosts)
+    return expected_hosts == {"{}:{}".format(*s) for s in actual.all_hosts}
 
 
 def compare_topology_descriptions(expected, actual):
@@ -60,7 +60,7 @@ def compare_topology_descriptions(expected, actual):
     if len(expected) != len(actual):
         return False
     for exp_server in expected:
-        for address, actual_server in actual.items():
+        for _address, actual_server in actual.items():
             if compare_server_descriptions(exp_server, actual_server):
                 break
         else:
@@ -79,42 +79,42 @@ def compare_events(expected_dict, actual):
     if expected_type == "server_opening_event":
         if not isinstance(actual, monitoring.ServerOpeningEvent):
             return False, "Expected ServerOpeningEvent, got %s" % (actual.__class__)
-        if not expected["address"] == "%s:%s" % actual.server_address:
+        if not expected["address"] == "{}:{}".format(*actual.server_address):
             return (
                 False,
                 "ServerOpeningEvent published with wrong address (expected"
-                " %s, got %s" % (expected["address"], actual.server_address),
+                " {}, got {}".format(expected["address"], actual.server_address),
             )
 
     elif expected_type == "server_description_changed_event":
 
         if not isinstance(actual, monitoring.ServerDescriptionChangedEvent):
             return (False, "Expected ServerDescriptionChangedEvent, got %s" % (actual.__class__))
-        if not expected["address"] == "%s:%s" % actual.server_address:
+        if not expected["address"] == "{}:{}".format(*actual.server_address):
             return (
                 False,
                 "ServerDescriptionChangedEvent has wrong address"
-                " (expected %s, got %s" % (expected["address"], actual.server_address),
+                " (expected {}, got {}".format(expected["address"], actual.server_address),
             )
 
         if not compare_server_descriptions(expected["newDescription"], actual.new_description):
-            return (False, "New ServerDescription incorrect in" " ServerDescriptionChangedEvent")
+            return (False, "New ServerDescription incorrect in ServerDescriptionChangedEvent")
         if not compare_server_descriptions(
             expected["previousDescription"], actual.previous_description
         ):
             return (
                 False,
-                "Previous ServerDescription incorrect in" " ServerDescriptionChangedEvent",
+                "Previous ServerDescription incorrect in ServerDescriptionChangedEvent",
             )
 
     elif expected_type == "server_closed_event":
         if not isinstance(actual, monitoring.ServerClosedEvent):
             return False, "Expected ServerClosedEvent, got %s" % (actual.__class__)
-        if not expected["address"] == "%s:%s" % actual.server_address:
+        if not expected["address"] == "{}:{}".format(*actual.server_address):
             return (
                 False,
                 "ServerClosedEvent published with wrong address"
-                " (expected %s, got %s" % (expected["address"], actual.server_address),
+                " (expected {}, got {}".format(expected["address"], actual.server_address),
             )
 
     elif expected_type == "topology_opening_event":
@@ -125,19 +125,19 @@ def compare_events(expected_dict, actual):
         if not isinstance(actual, monitoring.TopologyDescriptionChangedEvent):
             return (
                 False,
-                "Expected TopologyDescriptionChangedEvent," " got %s" % (actual.__class__),
+                "Expected TopologyDescriptionChangedEvent, got %s" % (actual.__class__),
             )
         if not compare_topology_descriptions(expected["newDescription"], actual.new_description):
             return (
                 False,
-                "New TopologyDescription incorrect in " "TopologyDescriptionChangedEvent",
+                "New TopologyDescription incorrect in TopologyDescriptionChangedEvent",
             )
         if not compare_topology_descriptions(
             expected["previousDescription"], actual.previous_description
         ):
             return (
                 False,
-                "Previous TopologyDescription incorrect in" " TopologyDescriptionChangedEvent",
+                "Previous TopologyDescription incorrect in TopologyDescriptionChangedEvent",
             )
 
     elif expected_type == "topology_closed_event":
@@ -145,7 +145,7 @@ def compare_events(expected_dict, actual):
             return False, "Expected TopologyClosedEvent, got %s" % (actual.__class__)
 
     else:
-        return False, "Incorrect event: expected %s, actual %s" % (expected_type, actual)
+        return False, f"Incorrect event: expected {expected_type}, actual {actual}"
 
     return True, ""
 
@@ -170,7 +170,7 @@ def compare_multiple_events(i, expected_results, actual_results):
 
 class TestAllScenarios(IntegrationTest):
     def setUp(self):
-        super(TestAllScenarios, self).setUp()
+        super().setUp()
         self.all_listener = ServerAndTopologyEventListener()
 
 
@@ -186,9 +186,9 @@ def create_test(scenario_def):
             def _run(self):
                 time.sleep(0.05)
 
-        m = single_client(
-            h=scenario_def["uri"],
-            p=27017,
+        m = MongoClient(
+            host=scenario_def["uri"],
+            port=27017,
             event_listeners=[self.all_listener],
             _monitor_class=NoopMonitor,
         )
@@ -199,7 +199,9 @@ def create_test(scenario_def):
                 for (source, response) in phase.get("responses", []):
                     source_address = clean_node(source)
                     topology.on_change(
-                        ServerDescription(source_address, Hello(response), round_trip_time=0)
+                        ServerDescription(
+                            address=source_address, hello=Hello(response), round_trip_time=0
+                        )
                     )
 
                 expected_results = phase["outcome"]["events"]
@@ -233,7 +235,7 @@ def create_test(scenario_def):
                 # Assert no extra events.
                 extra_events = self.all_listener.results[expected_len:]
                 if extra_events:
-                    self.fail("Extra events %r" % (extra_events,))
+                    self.fail(f"Extra events {extra_events!r}")
 
                 self.all_listener.reset()
         finally:
@@ -249,7 +251,7 @@ def create_tests():
                 scenario_def = json.load(scenario_stream, object_hook=object_hook)
             # Construct test from scenario.
             new_test = create_test(scenario_def)
-            test_name = "test_%s" % (os.path.splitext(filename)[0],)
+            test_name = f"test_{os.path.splitext(filename)[0]}"
             new_test.__name__ = test_name
             setattr(TestAllScenarios, new_test.__name__, new_test)
 
@@ -258,10 +260,15 @@ create_tests()
 
 
 class TestSdamMonitoring(IntegrationTest):
+    knobs: client_knobs
+    listener: ServerAndTopologyEventListener
+    test_client: MongoClient
+    coll: Collection
+
     @classmethod
     @client_context.require_failCommand_fail_point
     def setUpClass(cls):
-        super(TestSdamMonitoring, cls).setUpClass()
+        super().setUpClass()
         # Speed up the tests by decreasing the event publish frequency.
         cls.knobs = client_knobs(events_queue_frequency=0.1)
         cls.knobs.enable()
@@ -277,7 +284,7 @@ class TestSdamMonitoring(IntegrationTest):
     def tearDownClass(cls):
         cls.test_client.close()
         cls.knobs.disable()
-        super(TestSdamMonitoring, cls).tearDownClass()
+        super().tearDownClass()
 
     def setUp(self):
         self.listener.reset()
@@ -334,7 +341,7 @@ class TestSdamMonitoring(IntegrationTest):
     def test_network_error_publishes_events(self):
         self._test_app_error({"closeConnection": True}, ConnectionFailure)
 
-    # In 4.4+, NotPrimary errors from failCommand don't cause SDAM state
+    # In 4.4+, not primary errors from failCommand don't cause SDAM state
     # changes because topologyVersion is not incremented.
     @client_context.require_version_max(4, 3)
     def test_not_primary_error_publishes_events(self):
