@@ -47,7 +47,7 @@ class TestSON(unittest.TestCase):
 
         self.assertEqual(a1, SON({"hello": "world"}))
         self.assertEqual(b2, SON((("hello", "world"), ("mike", "awesome"), ("hello_", "mike"))))
-        self.assertEqual(b2, dict((("hello_", "mike"), ("mike", "awesome"), ("hello", "world"))))
+        self.assertEqual(b2, {"hello_": "mike", "mike": "awesome", "hello": "world"})
 
         self.assertNotEqual(a1, b2)
         self.assertNotEqual(b2, SON((("hello_", "mike"), ("mike", "awesome"), ("hello", "world"))))
@@ -55,7 +55,7 @@ class TestSON(unittest.TestCase):
         # Explicitly test inequality
         self.assertFalse(a1 != SON({"hello": "world"}))
         self.assertFalse(b2 != SON((("hello", "world"), ("mike", "awesome"), ("hello_", "mike"))))
-        self.assertFalse(b2 != dict((("hello_", "mike"), ("mike", "awesome"), ("hello", "world"))))
+        self.assertFalse(b2 != {"hello_": "mike", "mike": "awesome", "hello": "world"})
 
         # Embedded SON.
         d4 = SON([("blah", {"foo": SON()})])
@@ -97,10 +97,10 @@ class TestSON(unittest.TestCase):
         # This string was generated by pickling a SON object in pymongo
         # version 2.1.1
         pickled_with_2_1_1 = (
-            "ccopy_reg\n_reconstructor\np0\n(cbson.son\nSON\np1\n"
-            "c__builtin__\ndict\np2\n(dp3\ntp4\nRp5\n(dp6\n"
-            "S'_SON__keys'\np7\n(lp8\nsb."
-        ).encode("utf8")
+            b"ccopy_reg\n_reconstructor\np0\n(cbson.son\nSON\np1\n"
+            b"c__builtin__\ndict\np2\n(dp3\ntp4\nRp5\n(dp6\n"
+            b"S'_SON__keys'\np7\n(lp8\nsb."
+        )
         son_2_1_1 = pickle.loads(pickled_with_2_1_1)
         self.assertEqual(son_2_1_1, SON([]))
 
@@ -138,29 +138,23 @@ class TestSON(unittest.TestCase):
         self.assertEqual(id(reflexive_son1), id(reflexive_son1["reflexive"]))
 
     def test_iteration(self):
-        """
-        Test __iter__
-        """
+        """Test __iter__"""
         # test success case
         test_son = SON([(1, 100), (2, 200), (3, 300)])
         for ele in test_son:
             self.assertEqual(ele * 100, test_son[ele])
 
     def test_contains_has(self):
-        """
-        has_key and __contains__
-        """
+        """has_key and __contains__"""
         test_son = SON([(1, 100), (2, 200), (3, 300)])
         self.assertIn(1, test_son)
         self.assertTrue(2 in test_son, "in failed")
         self.assertFalse(22 in test_son, "in succeeded when it shouldn't")
-        self.assertTrue(test_son.has_key(2), "has_key failed")
-        self.assertFalse(test_son.has_key(22), "has_key succeeded when it shouldn't")
+        self.assertTrue(test_son.has_key(2), "has_key failed")  # noqa
+        self.assertFalse(test_son.has_key(22), "has_key succeeded when it shouldn't")  # noqa
 
     def test_clears(self):
-        """
-        Test clear()
-        """
+        """Test clear()"""
         test_son = SON([(1, 100), (2, 200), (3, 300)])
         test_son.clear()
         self.assertNotIn(1, test_son)
@@ -169,9 +163,7 @@ class TestSON(unittest.TestCase):
         self.assertEqual({}, test_son.to_dict())
 
     def test_len(self):
-        """
-        Test len
-        """
+        """Test len"""
         test_son = SON()
         self.assertEqual(0, len(test_son))
         test_son = SON([(1, 100), (2, 200), (3, 300)])
