@@ -17,6 +17,7 @@
 import os
 import socket
 import sys
+from typing import Any
 
 sys.path[0:0] = [""]
 
@@ -39,18 +40,18 @@ from pymongo.write_concern import WriteConcern
 _HAVE_PYOPENSSL = False
 try:
     # All of these must be available to use PyOpenSSL
-    import OpenSSL
-    import requests
-    import service_identity
+    import OpenSSL  # noqa
+    import requests  # noqa
+    import service_identity  # noqa
 
     # Ensure service_identity>=18.1 is installed
-    from service_identity.pyopenssl import verify_ip_address
+    from service_identity.pyopenssl import verify_ip_address  # noqa
 
     from pymongo.ocsp_support import _load_trusted_ca_certs
 
     _HAVE_PYOPENSSL = True
 except ImportError:
-    _load_trusted_ca_certs = None
+    _load_trusted_ca_certs = None  # type: ignore
 
 
 if HAVE_SSL:
@@ -64,8 +65,6 @@ CA_BUNDLE_PEM = os.path.join(CERT_PATH, "trusted-ca.pem")
 CRL_PEM = os.path.join(CERT_PATH, "crl.pem")
 MONGODB_X509_USERNAME = "C=US,ST=New York,L=New York City,O=MDB,OU=Drivers,CN=client"
 
-_PY37PLUS = sys.version_info[:2] >= (3, 7)
-
 # To fully test this start a mongod instance (built with SSL support) like so:
 # mongod --dbpath /path/to/data/directory --sslOnNormalPorts \
 # --sslPEMKeyFile /path/to/pymongo/test/certificates/server.pem \
@@ -78,9 +77,7 @@ _PY37PLUS = sys.version_info[:2] >= (3, 7)
 
 
 class TestClientSSL(unittest.TestCase):
-    @unittest.skipIf(
-        HAVE_SSL, "The ssl module is available, can't test what " "happens without it."
-    )
+    @unittest.skipIf(HAVE_SSL, "The ssl module is available, can't test what happens without it.")
     def test_no_ssl_module(self):
         # Explicit
         self.assertRaises(ConfigurationError, MongoClient, ssl=True)
@@ -131,6 +128,8 @@ class TestClientSSL(unittest.TestCase):
 
 
 class TestSSL(IntegrationTest):
+    saved_port: int
+
     def assertClientWorks(self, client):
         coll = client.pymongo_test.ssl_test.with_options(
             write_concern=WriteConcern(w=client_context.w)
@@ -143,7 +142,7 @@ class TestSSL(IntegrationTest):
     @classmethod
     @unittest.skipUnless(HAVE_SSL, "The ssl module is not available.")
     def setUpClass(cls):
-        super(TestSSL, cls).setUpClass()
+        super().setUpClass()
         # MongoClient should connect to the primary by default.
         cls.saved_port = MongoClient.PORT
         MongoClient.PORT = client_context.port
@@ -151,7 +150,7 @@ class TestSSL(IntegrationTest):
     @classmethod
     def tearDownClass(cls):
         MongoClient.PORT = cls.saved_port
-        super(TestSSL, cls).tearDownClass()
+        super().tearDownClass()
 
     @client_context.require_tls
     def test_simple_ssl(self):
@@ -175,7 +174,7 @@ class TestSSL(IntegrationTest):
                 tlsCertificateKeyFile=CLIENT_ENCRYPTED_PEM,
                 tlsCertificateKeyFilePassword="qwerty",
                 tlsCAFile=CA_PEM,
-                serverSelectionTimeoutMS=100,
+                serverSelectionTimeoutMS=1000,
             )
         else:
             connected(
@@ -186,7 +185,7 @@ class TestSSL(IntegrationTest):
                     tlsCertificateKeyFilePassword="qwerty",
                     tlsCAFile=CA_PEM,
                     serverSelectionTimeoutMS=5000,
-                    **self.credentials
+                    **self.credentials  # type: ignore[arg-type]
                 )
             )
 
@@ -195,7 +194,9 @@ class TestSSL(IntegrationTest):
                 "&tlsCertificateKeyFile=%s&tlsCertificateKeyFilePassword=qwerty"
                 "&tlsCAFile=%s&serverSelectionTimeoutMS=5000"
             )
-            connected(MongoClient(uri_fmt % (CLIENT_ENCRYPTED_PEM, CA_PEM), **self.credentials))
+            connected(
+                MongoClient(uri_fmt % (CLIENT_ENCRYPTED_PEM, CA_PEM), **self.credentials)  # type: ignore[arg-type]
+            )
 
     @client_context.require_tlsCertificateKeyFile
     @client_context.require_no_auth
@@ -303,10 +304,7 @@ class TestSSL(IntegrationTest):
         ctx = get_ssl_context(None, None, None, None, False, True, False)
         self.assertFalse(ctx.check_hostname)
         ctx = get_ssl_context(None, None, None, None, False, False, False)
-        if _PY37PLUS or _HAVE_PYOPENSSL:
-            self.assertTrue(ctx.check_hostname)
-        else:
-            self.assertFalse(ctx.check_hostname)
+        self.assertTrue(ctx.check_hostname)
 
         response = self.client.admin.command(HelloCompat.LEGACY_CMD)
 
@@ -319,7 +317,7 @@ class TestSSL(IntegrationTest):
                     tlsAllowInvalidCertificates=False,
                     tlsCAFile=CA_PEM,
                     serverSelectionTimeoutMS=500,
-                    **self.credentials
+                    **self.credentials  # type: ignore[arg-type]
                 )
             )
 
@@ -332,7 +330,7 @@ class TestSSL(IntegrationTest):
                 tlsCAFile=CA_PEM,
                 tlsAllowInvalidHostnames=True,
                 serverSelectionTimeoutMS=500,
-                **self.credentials
+                **self.credentials  # type: ignore[arg-type]
             )
         )
 
@@ -347,7 +345,7 @@ class TestSSL(IntegrationTest):
                         tlsAllowInvalidCertificates=False,
                         tlsCAFile=CA_PEM,
                         serverSelectionTimeoutMS=500,
-                        **self.credentials
+                        **self.credentials  # type: ignore[arg-type]
                     )
                 )
 
@@ -361,7 +359,7 @@ class TestSSL(IntegrationTest):
                     tlsCAFile=CA_PEM,
                     tlsAllowInvalidHostnames=True,
                     serverSelectionTimeoutMS=500,
-                    **self.credentials
+                    **self.credentials  # type: ignore[arg-type]
                 )
             )
 
@@ -376,7 +374,7 @@ class TestSSL(IntegrationTest):
                 ssl=True,
                 tlsCAFile=CA_PEM,
                 tlsCRLFile=CRL_PEM,
-                serverSelectionTimeoutMS=100,
+                serverSelectionTimeoutMS=1000,
             )
         else:
             connected(
@@ -384,8 +382,8 @@ class TestSSL(IntegrationTest):
                     "localhost",
                     ssl=True,
                     tlsCAFile=CA_PEM,
-                    serverSelectionTimeoutMS=100,
-                    **self.credentials
+                    serverSelectionTimeoutMS=1000,
+                    **self.credentials  # type: ignore[arg-type]
                 )
             )
 
@@ -396,20 +394,22 @@ class TestSSL(IntegrationTest):
                         ssl=True,
                         tlsCAFile=CA_PEM,
                         tlsCRLFile=CRL_PEM,
-                        serverSelectionTimeoutMS=100,
-                        **self.credentials
+                        serverSelectionTimeoutMS=1000,
+                        **self.credentials  # type: ignore[arg-type]
                     )
                 )
 
-            uri_fmt = "mongodb://localhost/?ssl=true&" "tlsCAFile=%s&serverSelectionTimeoutMS=100"
-            connected(MongoClient(uri_fmt % (CA_PEM,), **self.credentials))
+            uri_fmt = "mongodb://localhost/?ssl=true&tlsCAFile=%s&serverSelectionTimeoutMS=1000"
+            connected(MongoClient(uri_fmt % (CA_PEM,), **self.credentials))  # type: ignore
 
             uri_fmt = (
                 "mongodb://localhost/?ssl=true&tlsCRLFile=%s"
-                "&tlsCAFile=%s&serverSelectionTimeoutMS=100"
+                "&tlsCAFile=%s&serverSelectionTimeoutMS=1000"
             )
             with self.assertRaises(ConnectionFailure):
-                connected(MongoClient(uri_fmt % (CRL_PEM, CA_PEM), **self.credentials))
+                connected(
+                    MongoClient(uri_fmt % (CRL_PEM, CA_PEM), **self.credentials)  # type: ignore[arg-type]
+                )
 
     @client_context.require_tlsCertificateKeyFile
     @client_context.require_server_resolvable
@@ -425,7 +425,7 @@ class TestSSL(IntegrationTest):
         with self.assertRaises(ConnectionFailure):
             # Server cert is verified but hostname matching fails
             connected(
-                MongoClient("server", ssl=True, serverSelectionTimeoutMS=100, **self.credentials)
+                MongoClient("server", ssl=True, serverSelectionTimeoutMS=1000, **self.credentials)  # type: ignore[arg-type]
             )
 
         # Server cert is verified. Disable hostname matching.
@@ -434,20 +434,20 @@ class TestSSL(IntegrationTest):
                 "server",
                 ssl=True,
                 tlsAllowInvalidHostnames=True,
-                serverSelectionTimeoutMS=100,
-                **self.credentials
+                serverSelectionTimeoutMS=1000,
+                **self.credentials  # type: ignore[arg-type]
             )
         )
 
         # Server cert and hostname are verified.
         connected(
-            MongoClient("localhost", ssl=True, serverSelectionTimeoutMS=100, **self.credentials)
+            MongoClient("localhost", ssl=True, serverSelectionTimeoutMS=1000, **self.credentials)  # type: ignore[arg-type]
         )
 
         # Server cert and hostname are verified.
         connected(
             MongoClient(
-                "mongodb://localhost/?ssl=true&serverSelectionTimeoutMS=100", **self.credentials
+                "mongodb://localhost/?ssl=true&serverSelectionTimeoutMS=1000", **self.credentials  # type: ignore[arg-type]
             )
         )
 
@@ -458,6 +458,7 @@ class TestSSL(IntegrationTest):
         ):
             raise SkipTest("Can't test when system CA certificates are loadable.")
 
+        ssl_support: Any
         have_certifi = ssl_support.HAVE_CERTIFI
         have_wincertstore = ssl_support.HAVE_WINCERTSTORE
         # Force the test regardless of environment.
@@ -476,6 +477,7 @@ class TestSSL(IntegrationTest):
             # with SSLContext and SSLContext provides no information
             # about ca_certs.
             raise SkipTest("Can't test when SSLContext available.")
+        ssl_support: Any
         if not ssl_support.HAVE_CERTIFI:
             raise SkipTest("Need certifi to test certifi support.")
 
@@ -560,7 +562,7 @@ class TestSSL(IntegrationTest):
         else:
             self.assertEqual(names, ["authenticate", "find"])
 
-        uri = "mongodb://%s@%s:%d/?authMechanism=" "MONGODB-X509" % (
+        uri = "mongodb://%s@%s:%d/?authMechanism=MONGODB-X509" % (
             quote_plus(MONGODB_X509_USERNAME),
             host,
             port,
@@ -580,7 +582,7 @@ class TestSSL(IntegrationTest):
         # No error
         client.pymongo_test.test.find_one()
         # Auth should fail if username and certificate do not match
-        uri = "mongodb://%s@%s:%d/?authMechanism=" "MONGODB-X509" % (
+        uri = "mongodb://%s@%s:%d/?authMechanism=MONGODB-X509" % (
             quote_plus("not the username"),
             host,
             port,
@@ -608,7 +610,7 @@ class TestSSL(IntegrationTest):
             bad_client.pymongo_test.test.find_one()
 
         # Invalid certificate (using CA certificate as client certificate)
-        uri = "mongodb://%s@%s:%d/?authMechanism=" "MONGODB-X509" % (
+        uri = "mongodb://%s@%s:%d/?authMechanism=MONGODB-X509" % (
             quote_plus(MONGODB_X509_USERNAME),
             host,
             port,
@@ -620,7 +622,7 @@ class TestSSL(IntegrationTest):
                     ssl=True,
                     tlsAllowInvalidCertificates=True,
                     tlsCertificateKeyFile=CA_PEM,
-                    serverSelectionTimeoutMS=100,
+                    serverSelectionTimeoutMS=1000,
                 )
             )
         except (ConnectionFailure, ConfigurationError):
