@@ -19,14 +19,13 @@ import sys
 sys.path[0:0] = [""]
 
 from test import client_knobs, unittest
+from test.pymongo_mocks import DummyMonitor
 from test.utils import MockPool, wait_until
 
 from bson.objectid import ObjectId
-from bson.py3compat import imap
 from pymongo import common
 from pymongo.errors import AutoReconnect, ConfigurationError, ConnectionFailure
-from pymongo.hello import Hello
-from pymongo.hello_compat import HelloCompat
+from pymongo.hello import Hello, HelloCompat
 from pymongo.monitor import Monitor
 from pymongo.pool import PoolOptions
 from pymongo.read_preferences import ReadPreference, Secondary
@@ -38,24 +37,6 @@ from pymongo.topology import Topology, _ErrorContext
 from pymongo.topology_description import TOPOLOGY_TYPE
 
 
-class MockMonitor(object):
-    def __init__(self, server_description, topology, pool, topology_settings):
-        self._server_description = server_description
-        self.opened = False
-
-    def cancel_check(self):
-        pass
-
-    def open(self):
-        self.opened = True
-
-    def request_check(self):
-        pass
-
-    def close(self):
-        self.opened = False
-
-
 class SetNameDiscoverySettings(TopologySettings):
     def get_topology_type(self):
         return TOPOLOGY_TYPE.ReplicaSetNoPrimary
@@ -64,13 +45,16 @@ class SetNameDiscoverySettings(TopologySettings):
 address = ("a", 27017)
 
 
-def create_mock_topology(seeds=None, replica_set_name=None, monitor_class=MockMonitor):
-    partitioned_seeds = list(imap(common.partition_node, seeds or ["a"]))
+def create_mock_topology(
+    seeds=None, replica_set_name=None, monitor_class=DummyMonitor, direct_connection=False
+):
+    partitioned_seeds = list(map(common.partition_node, seeds or ["a"]))
     topology_settings = TopologySettings(
         partitioned_seeds,
         replica_set_name=replica_set_name,
         pool_class=MockPool,
         monitor_class=monitor_class,
+        direct_connection=direct_connection,
     )
 
     t = Topology(topology_settings)
@@ -105,7 +89,7 @@ class TopologyTest(unittest.TestCase):
     """Disables periodic monitoring, to make tests deterministic."""
 
     def setUp(self):
-        super(TopologyTest, self).setUp()
+        super().setUp()
         self.client_knobs = client_knobs(heartbeat_frequency=999999)
         self.client_knobs.enable()
         self.addCleanup(self.client_knobs.disable)
@@ -175,11 +159,12 @@ class TestSingleServerTopology(TopologyTest):
                 },
             ),
             (SERVER_TYPE.Standalone, {"ok": 1, HelloCompat.LEGACY_CMD: True, "maxWireVersion": 6}),
-            # mongod started as "secondary" in the replication system
-            # that existed before replica sets.
+            # A "slave" in a master-slave deployment.
+            # This replication type was removed in MongoDB
+            # 4.0.
             (SERVER_TYPE.Standalone, {"ok": 1, HelloCompat.LEGACY_CMD: False, "maxWireVersion": 6}),
         ]:
-            t = create_mock_topology()
+            t = create_mock_topology(direct_connection=True)
 
             # Can't select a server while the only server is of type Unknown.
             with self.assertRaisesRegex(ConnectionFailure, "No servers found yet"):
@@ -393,7 +378,7 @@ class TestMultiServerTopology(TopologyTest):
         self.assertEqual("rs", t.description.replica_set_name)
         self.assertEqual(TOPOLOGY_TYPE.ReplicaSetNoPrimary, t.description.topology_type)
 
-        # A closed topology should not be updated when receiving an hello.
+        # A closed topology should not be updated when receiving a hello.
         got_hello(
             t,
             ("a", 27017),
@@ -452,41 +437,6 @@ class TestMultiServerTopology(TopologyTest):
         self.assertEqual("rs", t.description.replica_set_name)
         self.assertEqual(TOPOLOGY_TYPE.ReplicaSetWithPrimary, t.description.topology_type)
 
-    def test_handle_getlasterror(self):
-        t = create_mock_topology(replica_set_name="rs")
-        got_hello(
-            t,
-            ("a", 27017),
-            {"ok": 1, HelloCompat.LEGACY_CMD: True, "setName": "rs", "hosts": ["a", "b"]},
-        )
-
-        got_hello(
-            t,
-            ("b", 27017),
-            {
-                "ok": 1,
-                HelloCompat.LEGACY_CMD: False,
-                "secondary": True,
-                "setName": "rs",
-                "hosts": ["a", "b"],
-            },
-        )
-
-        t.handle_getlasterror(("a", 27017), HelloCompat.LEGACY_ERROR)
-        self.assertEqual(SERVER_TYPE.Unknown, get_type(t, "a"))
-        self.assertEqual(SERVER_TYPE.RSSecondary, get_type(t, "b"))
-        self.assertEqual("rs", t.description.replica_set_name)
-        self.assertEqual(TOPOLOGY_TYPE.ReplicaSetNoPrimary, t.description.topology_type)
-
-        got_hello(
-            t,
-            ("a", 27017),
-            {"ok": 1, HelloCompat.LEGACY_CMD: True, "setName": "rs", "hosts": ["a", "b"]},
-        )
-
-        self.assertEqual(SERVER_TYPE.RSPrimary, get_type(t, "a"))
-        self.assertEqual(TOPOLOGY_TYPE.ReplicaSetWithPrimary, t.description.topology_type)
-
     def test_handle_error_removed_server(self):
         t = create_mock_topology(replica_set_name="rs")
 
@@ -497,20 +447,11 @@ class TestMultiServerTopology(TopologyTest):
         # Server was *not* added as type Unknown.
         self.assertFalse(t.has_server(("b", 27017)))
 
-    def test_handle_getlasterror_removed_server(self):
-        t = create_mock_topology(replica_set_name="rs")
-
-        # No error resetting a server not in the TopologyDescription.
-        t.handle_getlasterror(("b", 27017), HelloCompat.LEGACY_ERROR)
-
-        # Server was *not* added as type Unknown.
-        self.assertFalse(t.has_server(("b", 27017)))
-
     def test_discover_set_name_from_primary(self):
         # Discovering a replica set without the setName supplied by the user
         # is not yet supported by MongoClient, but Topology can do it.
         topology_settings = SetNameDiscoverySettings(
-            seeds=[address], pool_class=MockPool, monitor_class=MockMonitor
+            seeds=[address], pool_class=MockPool, monitor_class=DummyMonitor
         )
 
         t = Topology(topology_settings)
@@ -538,7 +479,7 @@ class TestMultiServerTopology(TopologyTest):
         # Discovering a replica set without the setName supplied by the user
         # is not yet supported by MongoClient, but Topology can do it.
         topology_settings = SetNameDiscoverySettings(
-            seeds=[address], pool_class=MockPool, monitor_class=MockMonitor
+            seeds=[address], pool_class=MockPool, monitor_class=DummyMonitor
         )
 
         t = Topology(topology_settings)
@@ -582,12 +523,13 @@ class TestMultiServerTopology(TopologyTest):
                 "setName": "rs",
                 "hosts": ["a"],
                 "minWireVersion": 1,
-                "maxWireVersion": 5,
+                "maxWireVersion": 6,
             },
         )
 
         self.assertEqual(server.description.min_wire_version, 1)
-        self.assertEqual(server.description.max_wire_version, 5)
+        self.assertEqual(server.description.max_wire_version, 6)
+        t.select_servers(any_server_selector)
 
         # Incompatible.
         got_hello(
@@ -598,8 +540,8 @@ class TestMultiServerTopology(TopologyTest):
                 HelloCompat.LEGACY_CMD: True,
                 "setName": "rs",
                 "hosts": ["a"],
-                "minWireVersion": 21,
-                "maxWireVersion": 22,
+                "minWireVersion": 22,
+                "maxWireVersion": 24,
             },
         )
 
@@ -609,7 +551,7 @@ class TestMultiServerTopology(TopologyTest):
             # Error message should say which server failed and why.
             self.assertEqual(
                 str(e),
-                "Server at a:27017 requires wire version 21, but this version "
+                "Server at a:27017 requires wire version 22, but this version "
                 "of PyMongo only supports up to %d." % (common.MAX_SUPPORTED_WIRE_VERSION,),
             )
         else:
@@ -705,13 +647,13 @@ class TestMultiServerTopology(TopologyTest):
         )
         self.assertEqual(
             repr(t.description),
-            "<TopologyDescription id: %s, "
+            "<TopologyDescription id: {}, "
             "topology_type: ReplicaSetWithPrimary, servers: ["
             "<ServerDescription ('a', 27017) server_type: RSPrimary, rtt: 0>, "
             "<ServerDescription ('b', 27017) server_type: Unknown,"
             " rtt: None>, "
             "<ServerDescription ('c', 27017) server_type: Unknown,"
-            " rtt: None>]>" % (t._topology_id,),
+            " rtt: None>]>".format(t._topology_id),
         )
 
     def test_unexpected_load_balancer(self):
@@ -792,7 +734,7 @@ class TestTopologyErrors(TopologyTest):
                 if hello_count[0] in (1, 3):
                     return Hello({"ok": 1, "maxWireVersion": 6}), 0
                 else:
-                    raise AutoReconnect("mock monitor error #%s" % (hello_count[0],))
+                    raise AutoReconnect(f"mock monitor error #{hello_count[0]}")
 
         t = create_mock_topology(monitor_class=TestMonitor)
         self.addCleanup(t.close)
