@@ -12,8 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""Utilities for testing pymongo
-"""
+"""Utilities for testing pymongo"""
 
 import contextlib
 import copy
@@ -29,15 +28,31 @@ import warnings
 from collections import abc, defaultdict
 from functools import partial
 from test import client_context, db_pwd, db_user
+from typing import Any, List
 
 from bson import json_util
 from bson.objectid import ObjectId
 from bson.son import SON
 from pymongo import MongoClient, monitoring, operations, read_preferences
 from pymongo.collection import ReturnDocument
+from pymongo.cursor import CursorType
 from pymongo.errors import ConfigurationError, OperationFailure
 from pymongo.hello import HelloCompat
-from pymongo.monitoring import _SENSITIVE_COMMANDS
+from pymongo.lock import _create_lock
+from pymongo.monitoring import (
+    _SENSITIVE_COMMANDS,
+    ConnectionCheckedInEvent,
+    ConnectionCheckedOutEvent,
+    ConnectionCheckOutFailedEvent,
+    ConnectionCheckOutStartedEvent,
+    ConnectionClosedEvent,
+    ConnectionCreatedEvent,
+    ConnectionReadyEvent,
+    PoolClearedEvent,
+    PoolClosedEvent,
+    PoolCreatedEvent,
+    PoolReadyEvent,
+)
 from pymongo.pool import _CancellationContext, _PoolGeneration
 from pymongo.read_concern import ReadConcern
 from pymongo.read_preferences import ReadPreference
@@ -49,7 +64,7 @@ from pymongo.write_concern import WriteConcern
 IMPOSSIBLE_WRITE_CONCERN = WriteConcern(w=50)
 
 
-class BaseListener(object):
+class BaseListener:
     def __init__(self):
         self.events = []
 
@@ -75,64 +90,92 @@ class BaseListener(object):
 
     def wait_for_event(self, event, count):
         """Wait for a number of events to be published, or fail."""
-        wait_until(lambda: self.event_count(event) >= count, "find %s %s event(s)" % (count, event))
+        wait_until(lambda: self.event_count(event) >= count, f"find {count} {event} event(s)")
 
 
 class CMAPListener(BaseListener, monitoring.ConnectionPoolListener):
     def connection_created(self, event):
+        assert isinstance(event, ConnectionCreatedEvent)
         self.add_event(event)
 
     def connection_ready(self, event):
+        assert isinstance(event, ConnectionReadyEvent)
         self.add_event(event)
 
     def connection_closed(self, event):
+        assert isinstance(event, ConnectionClosedEvent)
         self.add_event(event)
 
     def connection_check_out_started(self, event):
+        assert isinstance(event, ConnectionCheckOutStartedEvent)
         self.add_event(event)
 
     def connection_check_out_failed(self, event):
+        assert isinstance(event, ConnectionCheckOutFailedEvent)
         self.add_event(event)
 
     def connection_checked_out(self, event):
+        assert isinstance(event, ConnectionCheckedOutEvent)
         self.add_event(event)
 
     def connection_checked_in(self, event):
+        assert isinstance(event, ConnectionCheckedInEvent)
         self.add_event(event)
 
     def pool_created(self, event):
+        assert isinstance(event, PoolCreatedEvent)
         self.add_event(event)
 
     def pool_ready(self, event):
+        assert isinstance(event, PoolReadyEvent)
         self.add_event(event)
 
     def pool_cleared(self, event):
+        assert isinstance(event, PoolClearedEvent)
         self.add_event(event)
 
     def pool_closed(self, event):
+        assert isinstance(event, PoolClosedEvent)
         self.add_event(event)
 
 
-class EventListener(monitoring.CommandListener):
+class EventListener(BaseListener, monitoring.CommandListener):
     def __init__(self):
+        super().__init__()
         self.results = defaultdict(list)
 
-    def started(self, event):
-        self.results["started"].append(event)
+    @property
+    def started_events(self) -> List[monitoring.CommandStartedEvent]:
+        return self.results["started"]
 
-    def succeeded(self, event):
-        self.results["succeeded"].append(event)
+    @property
+    def succeeded_events(self) -> List[monitoring.CommandSucceededEvent]:
+        return self.results["succeeded"]
 
-    def failed(self, event):
-        self.results["failed"].append(event)
+    @property
+    def failed_events(self) -> List[monitoring.CommandFailedEvent]:
+        return self.results["failed"]
 
-    def started_command_names(self):
+    def started(self, event: monitoring.CommandStartedEvent) -> None:
+        self.started_events.append(event)
+        self.add_event(event)
+
+    def succeeded(self, event: monitoring.CommandSucceededEvent) -> None:
+        self.succeeded_events.append(event)
+        self.add_event(event)
+
+    def failed(self, event: monitoring.CommandFailedEvent) -> None:
+        self.failed_events.append(event)
+        self.add_event(event)
+
+    def started_command_names(self) -> List[str]:
         """Return list of command names started."""
-        return [event.command_name for event in self.results["started"]]
+        return [event.command_name for event in self.started_events]
 
-    def reset(self):
+    def reset(self) -> None:
         """Reset the state of this listener."""
         self.results.clear()
+        super().reset()
 
 
 class TopologyEventListener(monitoring.TopologyListener):
@@ -156,38 +199,40 @@ class TopologyEventListener(monitoring.TopologyListener):
 class AllowListEventListener(EventListener):
     def __init__(self, *commands):
         self.commands = set(commands)
-        super(AllowListEventListener, self).__init__()
+        super().__init__()
 
     def started(self, event):
         if event.command_name in self.commands:
-            super(AllowListEventListener, self).started(event)
+            super().started(event)
 
     def succeeded(self, event):
         if event.command_name in self.commands:
-            super(AllowListEventListener, self).succeeded(event)
+            super().succeeded(event)
 
     def failed(self, event):
         if event.command_name in self.commands:
-            super(AllowListEventListener, self).failed(event)
+            super().failed(event)
 
 
 class OvertCommandListener(EventListener):
     """A CommandListener that ignores sensitive commands."""
 
+    ignore_list_collections = False
+
     def started(self, event):
         if event.command_name.lower() not in _SENSITIVE_COMMANDS:
-            super(OvertCommandListener, self).started(event)
+            super().started(event)
 
     def succeeded(self, event):
         if event.command_name.lower() not in _SENSITIVE_COMMANDS:
-            super(OvertCommandListener, self).succeeded(event)
+            super().succeeded(event)
 
     def failed(self, event):
         if event.command_name.lower() not in _SENSITIVE_COMMANDS:
-            super(OvertCommandListener, self).failed(event)
+            super().failed(event)
 
 
-class _ServerEventListener(object):
+class _ServerEventListener:
     """Listens to all events."""
 
     def __init__(self):
@@ -215,7 +260,9 @@ class ServerEventListener(_ServerEventListener, monitoring.ServerListener):
     """Listens to Server events."""
 
 
-class ServerAndTopologyEventListener(ServerEventListener, monitoring.TopologyListener):
+class ServerAndTopologyEventListener(  # type: ignore[misc]
+    ServerEventListener, monitoring.TopologyListener
+):
     """Listens to Server and Topology events."""
 
 
@@ -232,7 +279,7 @@ class HeartbeatEventListener(BaseListener, monitoring.ServerHeartbeatListener):
         self.add_event(event)
 
 
-class MockSocketInfo(object):
+class MockSocketInfo:
     def __init__(self):
         self.cancel_context = _CancellationContext()
         self.more_to_come = False
@@ -247,17 +294,17 @@ class MockSocketInfo(object):
         pass
 
 
-class MockPool(object):
+class MockPool:
     def __init__(self, address, options, handshake=True):
         self.gen = _PoolGeneration()
-        self._lock = threading.Lock()
+        self._lock = _create_lock()
         self.opts = options
         self.operation_count = 0
 
     def stale_generation(self, gen, service_id):
         return self.gen.stale(gen, service_id)
 
-    def get_socket(self, all_credentials, handler=None):
+    def get_socket(self, handler=None):
         return MockSocketInfo()
 
     def return_socket(self, *args, **kwargs):
@@ -309,21 +356,17 @@ class ScenarioDict(dict):
             return ScenarioDict({})
 
 
-class CompareType(object):
-    """Class that compares equal to any object of the given type."""
+class CompareType:
+    """Class that compares equal to any object of the given type(s)."""
 
-    def __init__(self, type):
-        self.type = type
+    def __init__(self, types):
+        self.types = types
 
     def __eq__(self, other):
-        return isinstance(other, self.type)
-
-    def __ne__(self, other):
-        """Needed for Python 2."""
-        return not self.__eq__(other)
+        return isinstance(other, self.types)
 
 
-class FunctionCallRecorder(object):
+class FunctionCallRecorder:
     """Utility class to wrap a callable and record its invocations."""
 
     def __init__(self, function):
@@ -348,7 +391,7 @@ class FunctionCallRecorder(object):
         return len(self._call_list)
 
 
-class TestCreator(object):
+class TestCreator:
     """Class to create test cases from specifications."""
 
     def __init__(self, create_test, test_class, test_path):
@@ -371,7 +414,8 @@ class TestCreator(object):
 
     def _ensure_min_max_server_version(self, scenario_def, method):
         """Test modifier that enforces a version range for the server on a
-        test case."""
+        test case.
+        """
         if "minServerVersion" in scenario_def:
             min_ver = tuple(int(elt) for elt in scenario_def["minServerVersion"].split("."))
             if min_ver is not None:
@@ -480,7 +524,7 @@ class TestCreator(object):
 
                 # Construct test from scenario.
                 for test_def in self.tests(scenario_def):
-                    test_name = "test_%s_%s_%s" % (
+                    test_name = "test_{}_{}_{}".format(
                         dirname,
                         test_type.replace("-", "_").replace(".", "_"),
                         str(test_def["description"].replace(" ", "_").replace(".", "_")),
@@ -495,16 +539,16 @@ class TestCreator(object):
 
 
 def _connection_string(h):
-    if h.startswith("mongodb://") or h.startswith("mongodb+srv://"):
+    if h.startswith(("mongodb://", "mongodb+srv://")):
         return h
-    return "mongodb://%s" % (str(h),)
+    return f"mongodb://{str(h)}"
 
 
 def _mongo_client(host, port, authenticate=True, directConnection=None, **kwargs):
     """Create a new client over SSL/TLS if necessary."""
     host = host or client_context.host
     port = port or client_context.port
-    client_options = client_context.default_client_options.copy()
+    client_options: dict = client_context.default_client_options.copy()
     if client_context.replica_set_name and not directConnection:
         client_options["replicaSet"] = client_context.replica_set_name
     if directConnection is not None:
@@ -527,27 +571,27 @@ def _mongo_client(host, port, authenticate=True, directConnection=None, **kwargs
     return MongoClient(uri, port, **client_options)
 
 
-def single_client_noauth(h=None, p=None, **kwargs):
+def single_client_noauth(h: Any = None, p: Any = None, **kwargs: Any) -> MongoClient[dict]:
     """Make a direct connection. Don't authenticate."""
     return _mongo_client(h, p, authenticate=False, directConnection=True, **kwargs)
 
 
-def single_client(h=None, p=None, **kwargs):
+def single_client(h: Any = None, p: Any = None, **kwargs: Any) -> MongoClient[dict]:
     """Make a direct connection, and authenticate if necessary."""
     return _mongo_client(h, p, directConnection=True, **kwargs)
 
 
-def rs_client_noauth(h=None, p=None, **kwargs):
+def rs_client_noauth(h: Any = None, p: Any = None, **kwargs: Any) -> MongoClient[dict]:
     """Connect to the replica set. Don't authenticate."""
     return _mongo_client(h, p, authenticate=False, **kwargs)
 
 
-def rs_client(h=None, p=None, **kwargs):
+def rs_client(h: Any = None, p: Any = None, **kwargs: Any) -> MongoClient[dict]:
     """Connect to the replica set and authenticate if necessary."""
     return _mongo_client(h, p, **kwargs)
 
 
-def rs_or_single_client_noauth(h=None, p=None, **kwargs):
+def rs_or_single_client_noauth(h: Any = None, p: Any = None, **kwargs: Any) -> MongoClient[dict]:
     """Connect to the replica set if there is one, otherwise the standalone.
 
     Like rs_or_single_client, but does not authenticate.
@@ -555,7 +599,7 @@ def rs_or_single_client_noauth(h=None, p=None, **kwargs):
     return _mongo_client(h, p, authenticate=False, **kwargs)
 
 
-def rs_or_single_client(h=None, p=None, **kwargs):
+def rs_or_single_client(h: Any = None, p: Any = None, **kwargs: Any) -> MongoClient[Any]:
     """Connect to the replica set if there is one, otherwise the standalone.
 
     Authenticates if necessary.
@@ -563,7 +607,7 @@ def rs_or_single_client(h=None, p=None, **kwargs):
     return _mongo_client(h, p, **kwargs)
 
 
-def ensure_all_connected(client):
+def ensure_all_connected(client: MongoClient) -> None:
     """Ensure that the client's connection pool has socket connections to all
     members of a replica set. Raises ConfigurationError when called with a
     non-replica set client.
@@ -571,18 +615,30 @@ def ensure_all_connected(client):
     Depending on the use-case, the caller may need to clear any event listeners
     that are configured on the client.
     """
-    hello = client.admin.command(HelloCompat.LEGACY_CMD)
+    hello: dict = client.admin.command(HelloCompat.LEGACY_CMD)
     if "setName" not in hello:
         raise ConfigurationError("cluster is not a replica set")
 
-    target_host_list = set(hello["hosts"])
-    connected_host_list = set([hello["me"]])
-    admindb = client.get_database("admin")
+    target_host_list = set(hello["hosts"] + hello.get("passives", []))
+    connected_host_list = {hello["me"]}
 
     # Run hello until we have connected to each host at least once.
-    while connected_host_list != target_host_list:
-        hello = admindb.command(HelloCompat.LEGACY_CMD, read_preference=ReadPreference.SECONDARY)
-        connected_host_list.update([hello["me"]])
+    def discover():
+        i = 0
+        while i < 100 and connected_host_list != target_host_list:
+            hello: dict = client.admin.command(
+                HelloCompat.LEGACY_CMD, read_preference=ReadPreference.SECONDARY
+            )
+            connected_host_list.update([hello["me"]])
+            i += 1
+        return connected_host_list
+
+    try:
+        wait_until(lambda: target_host_list == discover(), "connected to all hosts")
+    except AssertionError as exc:
+        raise AssertionError(
+            f"{exc}, {connected_host_list} != {target_host_list}, {client.topology_description}"
+        )
 
 
 def one(s):
@@ -638,6 +694,9 @@ def parse_collection_options(opts):
 
     if "readConcern" in opts:
         opts["read_concern"] = ReadConcern(**dict(opts.pop("readConcern")))
+
+    if "timeoutMS" in opts:
+        opts["timeout"] = int(opts.pop("timeoutMS")) / 1000.0
     return opts
 
 
@@ -661,7 +720,7 @@ def server_started_with_auth(client):
     try:
         command_line = get_command_line(client)
     except OperationFailure as e:
-        msg = e.details.get("errmsg", "")
+        msg = e.details.get("errmsg", "")  # type: ignore
         if e.code == 13 or "unauthorized" in msg or "login" in msg:
             # Unauthorized.
             return True
@@ -762,7 +821,7 @@ def assertRaisesExactly(cls, fn, *args, **kwargs):
     try:
         fn(*args, **kwargs)
     except Exception as e:
-        assert e.__class__ == cls, "got %s, expected %s" % (e.__class__.__name__, cls.__name__)
+        assert e.__class__ == cls, f"got {e.__class__.__name__}, expected {cls.__name__}"
     else:
         raise AssertionError("%s not raised" % cls)
 
@@ -789,7 +848,7 @@ def ignore_deprecations(wrapped=None):
         return _ignore_deprecations()
 
 
-class DeprecationFilter(object):
+class DeprecationFilter:
     def __init__(self, action="ignore"):
         """Start filtering deprecations."""
         self.warn_context = warnings.catch_warnings()
@@ -798,8 +857,8 @@ class DeprecationFilter(object):
 
     def stop(self):
         """Stop filtering deprecations."""
-        self.warn_context.__exit__()
-        self.warn_context = None
+        self.warn_context.__exit__()  # type: ignore
+        self.warn_context = None  # type: ignore
 
 
 def get_pool(client):
@@ -840,23 +899,13 @@ def run_threads(collection, target):
 @contextlib.contextmanager
 def frequent_thread_switches():
     """Make concurrency bugs more likely to manifest."""
-    interval = None
-    if not sys.platform.startswith("java"):
-        if hasattr(sys, "getswitchinterval"):
-            interval = sys.getswitchinterval()
-            sys.setswitchinterval(1e-6)
-        else:
-            interval = sys.getcheckinterval()
-            sys.setcheckinterval(1)
+    interval = sys.getswitchinterval()
+    sys.setswitchinterval(1e-6)
 
     try:
         yield
     finally:
-        if not sys.platform.startswith("java"):
-            if hasattr(sys, "setswitchinterval"):
-                sys.setswitchinterval(interval)
-            else:
-                sys.setcheckinterval(interval)
+        sys.setswitchinterval(interval)
 
 
 def lazy_client_trial(reset, target, test, get_client):
@@ -873,7 +922,7 @@ def lazy_client_trial(reset, target, test, get_client):
     collection = client_context.client.pymongo_test.test
 
     with frequent_thread_switches():
-        for i in range(NTRIALS):
+        for _i in range(NTRIALS):
             reset(collection)
             lazy_client = get_client()
             lazy_collection = lazy_client.pymongo_test.test
@@ -883,17 +932,14 @@ def lazy_client_trial(reset, target, test, get_client):
 
 def gevent_monkey_patched():
     """Check if gevent's monkey patching is active."""
-    # In Python 3.6 importing gevent.socket raises an ImportWarning.
-    with warnings.catch_warnings():
-        warnings.simplefilter("ignore", ImportWarning)
-        try:
-            import socket
+    try:
+        import socket
 
-            import gevent.socket
+        import gevent.socket
 
-            return socket.socket is gevent.socket.socket
-        except ImportError:
-            return False
+        return socket.socket is gevent.socket.socket
+    except ImportError:
+        return False
 
 
 def eventlet_monkey_patched():
@@ -908,14 +954,14 @@ def is_greenthread_patched():
 
 
 def disable_replication(client):
-    """Disable replication on all secondaries, requires MongoDB 3.2."""
+    """Disable replication on all secondaries."""
     for host, port in client.secondaries:
         secondary = single_client(host, port)
         secondary.admin.command("configureFailPoint", "stopReplProducer", mode="alwaysOn")
 
 
 def enable_replication(client):
-    """Enable replication on all secondaries, requires MongoDB 3.2."""
+    """Enable replication on all secondaries."""
     for host, port in client.secondaries:
         secondary = single_client(host, port)
         secondary.admin.command("configureFailPoint", "stopReplProducer", mode="off")
@@ -926,11 +972,11 @@ class ExceptionCatchingThread(threading.Thread):
 
     def __init__(self, *args, **kwargs):
         self.exc = None
-        super(ExceptionCatchingThread, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def run(self):
         try:
-            super(ExceptionCatchingThread, self).run()
+            super().run()
         except BaseException as exc:
             self.exc = exc
             raise
@@ -972,9 +1018,7 @@ def assertion_context(msg):
     try:
         yield
     except AssertionError as exc:
-        msg = "%s (%s)" % (exc, msg)
-        exc_type, exc_val, exc_tb = sys.exc_info()
-        raise exc_type(exc_val).with_traceback(exc_tb)
+        raise AssertionError(f"{msg}: {exc}")
 
 
 def parse_spec_options(opts):
@@ -987,6 +1031,10 @@ def parse_spec_options(opts):
     if "readConcern" in opts:
         opts["read_concern"] = ReadConcern(**dict(opts.pop("readConcern")))
 
+    if "timeoutMS" in opts:
+        assert isinstance(opts["timeoutMS"], int)
+        opts["timeout"] = int(opts.pop("timeoutMS")) / 1000.0
+
     if "maxTimeMS" in opts:
         opts["max_time_ms"] = opts.pop("maxTimeMS")
 
@@ -1038,6 +1086,8 @@ def prepare_spec_arguments(spec, arguments, opname, entity_map, with_txn_callbac
         # Aggregate uses "batchSize", while find uses batch_size.
         elif (arg_name == "batchSize" or arg_name == "allowDiskUse") and opname == "aggregate":
             continue
+        elif arg_name == "timeoutMode":
+            raise unittest.SkipTest("PyMongo does not support timeoutMode")
         # Requires boolean returnDocument.
         elif arg_name == "returnDocument":
             arguments[c2s] = getattr(ReturnDocument, arguments.pop(arg_name).upper())
@@ -1059,14 +1109,9 @@ def prepare_spec_arguments(spec, arguments, opname, entity_map, with_txn_callbac
             arguments["requests"] = requests
         elif arg_name == "session":
             arguments["session"] = entity_map[arguments["session"]]
-        elif opname in ("command", "run_admin_command") and arg_name == "command":
-            # Ensure the first key is the command name.
-            ordered_command = SON([(spec["command_name"], 1)])
-            ordered_command.update(arguments["command"])
-            arguments["command"] = ordered_command
         elif opname == "open_download_stream" and arg_name == "id":
             arguments["file_id"] = arguments.pop(arg_name)
-        elif opname != "find" and c2s == "max_time_ms":
+        elif opname not in ("find", "find_one") and c2s == "max_time_ms":
             # find is the only method that accepts snake_case max_time_ms.
             # All other methods take kwargs which must use the server's
             # camelCase maxTimeMS. See PYTHON-1855.
@@ -1084,11 +1129,24 @@ def prepare_spec_arguments(spec, arguments, opname, entity_map, with_txn_callbac
         elif opname == "create_collection":
             if arg_name == "collection":
                 arguments["name"] = arguments.pop(arg_name)
+            arguments["check_exists"] = False
             # Any other arguments to create_collection are passed through
             # **kwargs.
         elif opname == "create_index" and arg_name == "keys":
             arguments["keys"] = list(arguments.pop(arg_name).items())
         elif opname == "drop_index" and arg_name == "name":
             arguments["index_or_name"] = arguments.pop(arg_name)
+        elif opname == "rename" and arg_name == "to":
+            arguments["new_name"] = arguments.pop(arg_name)
+        elif opname == "rename" and arg_name == "dropTarget":
+            arguments["dropTarget"] = arguments.pop(arg_name)
+        elif arg_name == "cursorType":
+            cursor_type = arguments.pop(arg_name)
+            if cursor_type == "tailable":
+                arguments["cursor_type"] = CursorType.TAILABLE
+            elif cursor_type == "tailableAwait":
+                arguments["cursor_type"] = CursorType.TAILABLE
+            else:
+                raise AssertionError(f"Unsupported cursorType: {cursor_type}")
         else:
             arguments[c2s] = arguments.pop(arg_name)
