@@ -24,18 +24,19 @@ import sys
 
 sys.path[0:0] = [""]
 
-import bson
-import pymongo
+import bson  # noqa: E402
+import pymongo  # noqa: E402
 
 if not pymongo.has_c() or not bson.has_c():
     sys.exit("could not load C extensions")
 
 if os.environ.get("ENSURE_UNIVERSAL2") == "1":
     parent_dir = os.path.dirname(pymongo.__path__[0])
-    for so_file in glob.glob("%s/**/*.so" % parent_dir):
-        print("Checking universal2 compatibility in %s..." % so_file)
-        output = subprocess.check_output(["file", so_file])
-        if "arm64" not in output.decode("utf-8"):
-            sys.exit("Universal wheel was not compiled with arm64 support")
-        if "x86_64" not in output.decode("utf-8"):
-            sys.exit("Universal wheel was not compiled with x86_64 support")
+    for pkg in ["pymongo", "bson", "grifs"]:
+        for so_file in glob.glob(f"{parent_dir}/{pkg}/*.so"):
+            print(f"Checking universal2 compatibility in {so_file}...")
+            output = subprocess.check_output(["file", so_file])
+            if "arm64" not in output.decode("utf-8"):
+                sys.exit("Universal wheel was not compiled with arm64 support")
+            if "x86_64" not in output.decode("utf-8"):
+                sys.exit("Universal wheel was not compiled with x86_64 support")
